@@ -1,6 +1,6 @@
 /*
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
- *                                                            1998, 2011
+ *                                                            1998, 2012
  *
  * SARG donations:
  *      please look at http://sarg.sourceforge.net/donations.php
@@ -68,15 +68,17 @@ void authfail_report(void)
 	ouser2[0]='\0';
 	oip[0]='\0';
 
-	snprintf(tmp4,sizeof(tmp4),"%s/authfail.log.unsort",tmp);
+	snprintf(tmp4,sizeof(tmp4),"%s/authfail.int_unsort",tmp);
 
 	if(authfail_count == 0) {
-		unlink(tmp4);
+		if (!KeepTempLog && unlink(tmp4)) {
+			debuga(_("Cannot delete \"%s\": %s\n"),tmp4,strerror(errno));
+		}
 		if (debugz) debugaz(_("Authentication failures report not produced because it is empty\n"));
 		return;
 	}
 
-	snprintf(authfail_in,sizeof(authfail_in),"%s/authfail.log",tmp);
+	snprintf(authfail_in,sizeof(authfail_in),"%s/authfail.int_log",tmp);
 	snprintf(report,sizeof(report),"%s/authfail.html",outdirname);
 
 	snprintf(csort,sizeof(csort),"sort -b -t \"\t\" -T \"%s\" -k 3,3 -k 5,5 -o \"%s\" \"%s\"", tmp, authfail_in, tmp4);
@@ -91,8 +93,8 @@ void authfail_report(void)
 		debuga(_("sort command: %s\n"),csort);
 		exit(EXIT_FAILURE);
 	}
-	if (unlink(tmp4)) {
-		debuga(_("Cannot delete %s - %s\n"),tmp4,strerror(errno));
+	if (!KeepTempLog && unlink(tmp4)) {
+		debuga(_("Cannot delete \"%s\": %s\n"),tmp4,strerror(errno));
 		exit(EXIT_FAILURE);
 	}
 
@@ -204,8 +206,8 @@ void authfail_report(void)
 	if (fclose(fp_ou)==EOF)
 		debuga(_("Failed to close file %s - %s\n"),report,strerror(errno));
 
-	if (unlink(authfail_in)) {
-		debuga(_("Cannot delete %s - %s\n"),authfail_in,strerror(errno));
+	if (!KeepTempLog && unlink(authfail_in)) {
+		debuga(_("Cannot delete \"%s\": %s\n"),authfail_in,strerror(errno));
 		exit(EXIT_FAILURE);
 	}
 
