@@ -1,6 +1,6 @@
 /*
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
- *                                                            1998, 2011
+ *                                                            1998, 2012
  *
  * SARG donations:
  *      please look at http://sarg.sourceforge.net/donations.php
@@ -42,7 +42,9 @@ void convlog(const char *arq, char *df, int dfrom, int duntil)
 	if(arq[0] == '\0')
 		arq="/var/log/squid/access.log";
 
-	if((fp_in=MY_FOPEN(arq,"r"))==NULL) {
+	if (arq[0]=='-' && arq[1]=='\0') {
+		fp_in=stdin;
+	} else if((fp_in=MY_FOPEN(arq,"r"))==NULL) {
 		debuga(_("(convlog) Cannot open log file %s - %s\n"),arq,strerror(errno));
 		exit(EXIT_FAILURE);
 	}
@@ -76,7 +78,7 @@ void convlog(const char *arq, char *df, int dfrom, int duntil)
 	}
 
 	longline_destroy(&line);
-	if (fclose(fp_in)==EOF) {
+	if (fp_in!=stdin && fclose(fp_in)==EOF) {
 		debuga(_("Failed to close file %s - %s\n"),arq,strerror(errno));
 	}
 }
