@@ -1,6 +1,6 @@
 /*
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
- *                                                            1998, 2011
+ *                                                            1998, 2012
  *
  * SARG donations:
  *      please look at http://sarg.sourceforge.net/donations.php
@@ -49,8 +49,8 @@ void dansguardian_log(void)
 	dfrom=(period.start.tm_year+1900)*10000+(period.start.tm_mon+1)*100+period.start.tm_mday;
 	duntil=(period.end.tm_year+1900)*10000+(period.end.tm_mon+1)*100+period.end.tm_mday;
 
-	snprintf(guard_in,sizeof(guard_in),"%s/dansguardian.unsort",tmp);
-	snprintf(guard_ou,sizeof(guard_ou),"%s/dansguardian.log",tmp);
+	snprintf(guard_in,sizeof(guard_in),"%s/dansguardian.int_unsort",tmp);
+	snprintf(guard_ou,sizeof(guard_ou),"%s/dansguardian.int_log",tmp);
 
 	if(access(DansGuardianConf, R_OK) != 0) {
 		debuga(_("Cannot open DansGuardian config file: %s\n"),DansGuardianConf);
@@ -154,8 +154,8 @@ void dansguardian_log(void)
 		debuga(_("sort command: %s\n"),tmp6);
 		exit(EXIT_FAILURE);
 	}
-	if (unlink(guard_in)) {
-		debuga(_("Cannot delete %s - %s\n"),guard_in,strerror(errno));
+	if (!KeepTempLog && unlink(guard_in)) {
+		debuga(_("Cannot delete \"%s\": %s\n"),guard_in,strerror(errno));
 		exit(EXIT_FAILURE);
 	}
 }
