@@ -1,6 +1,6 @@
 /*
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
- *                                                            1998, 2011
+ *                                                            1998, 2012
  *
  * SARG donations:
  *      please look at http://sarg.sourceforge.net/donations.php
@@ -58,9 +58,10 @@ void dansguardian_report(void)
 
 	ouser[0]='\0';
 
-	snprintf(dansguardian_in,sizeof(dansguardian_in),"%s/dansguardian.log",tmp);
+	snprintf(dansguardian_in,sizeof(dansguardian_in),"%s/dansguardian.int_log",tmp);
 	if(!dansguardian_count) {
-		unlink(dansguardian_in);
+		if (!KeepTempLog && unlink(dansguardian_in))
+			debuga(_("Cannot delete \"%s\": %s\n"),dansguardian_in,strerror(errno));
 		if (debugz) debugaz(_("Dansguardian report not generated because it is empty\n"));
 		return;
 	}
@@ -169,8 +170,8 @@ void dansguardian_report(void)
 	if (fclose(fp_ou)==EOF)
 		debuga(_("Failed to close file %s - %s\n"),report,strerror(errno));
 
-	if (unlink(dansguardian_in)) {
-		debuga(_("Cannot delete %s - %s\n"),dansguardian_in,strerror(errno));
+	if (!KeepTempLog && unlink(dansguardian_in)) {
+		debuga(_("Cannot delete \"%s\": %s\n"),dansguardian_in,strerror(errno));
 		exit(EXIT_FAILURE);
 	}
 
