@@ -1,6 +1,6 @@
 /*
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
- *                                                            1998, 2011
+ *                                                            1998, 2012
  *
  * SARG donations:
  *      please look at http://sarg.sourceforge.net/donations.php
@@ -82,8 +82,9 @@ void data_file(char *tmp)
 		user_find(uinfo->label,MAX_USER_LEN, u2);
 		if(debug) debuga(_("Reading user file: %s/%s\n"),tmp,uinfo->filename);
 
-		if (snprintf(tmp3,sizeof(tmp3),"%s/%s.log",tmp,uinfo->filename)>=sizeof(tmp3)) {
-			debuga(_("(datafile) directory path too long: %s/%s.log\n"),tmp,uinfo->filename);
+		sort_users_log(tmp,debug,uinfo);
+		if (snprintf(tmp3,sizeof(tmp3),"%s/%s.user_log",tmp,uinfo->filename)>=sizeof(tmp3)) {
+			debuga(_("(datafile) directory path too long: %s/%s.user_log\n"),tmp,uinfo->filename);
 			exit(EXIT_FAILURE);
 		}
 
