@@ -1,6 +1,6 @@
 /*
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
- *                                                            1998, 2011
+ *                                                            1998, 2012
  *
  * SARG donations:
  *      please look at http://sarg.sourceforge.net/donations.php
@@ -62,9 +62,10 @@ void gen_denied_report(void)
 	ouser[0]='\0';
 	ouser2[0]='\0';
 
-	sprintf(denied_in,"%s/denied.log",tmp);
+	sprintf(denied_in,"%s/denied.int_log",tmp);
 	if (!denied_count) {
-		unlink(denied_in);
+		if (!KeepTempLog && unlink(denied_in))
+			debuga(_("Cannot delete \"%s\": %s\n"),denied_in,strerror(errno));
 		if (debugz) debugaz(_("Denied report not produced because it is empty\n"));
 		return;
 	}
@@ -148,9 +149,12 @@ void gen_denied_report(void)
 		}
 
 		fputs("<tr>",fp_ou);
-		if (new_user)
-			fprintf(fp_ou,"<td class=\"data\"><a href=\"%s/%s.html\">%s</a></td><td class=\"data\">%s</td>",uinfo->filename,uinfo->filename,uinfo->label,ip);
-		else
+		if (new_user) {
+			if (uinfo->topuser)
+				fprintf(fp_ou,"<td class=\"data\"><a href=\"%s/%s.html\">%s</a></td><td class=\"data\">%s</td>",uinfo->filename,uinfo->filename,uinfo->label,ip);
+			else
+				fprintf(fp_ou,"<td class=\"data\">%s</td><td class=\"data\">%s</td>",uinfo->label,ip);
+		} else
 			fputs("<td class=\"data\"></td><td class=\"data\"></td>",fp_ou);
 		fprintf(fp_ou,"<td class=\"data\">%s-%s</td><td class=\"data2\">",data,hora);
 		if(BlockIt[0] != '\0' && url[0]!=ALIAS_PREFIX) {
@@ -173,8 +177,8 @@ void gen_denied_report(void)
 	if (fclose(fp_ou)<0)
 		debuga(_("Failed to close file %s - %s\n"),report,strerror(errno));
 
-	if (unlink(denied_in)==-1)
-		debuga(_("Failed to delete the file \"%s\" after processing it - %s\n"),denied_in,strerror(errno));
+	if (!KeepTempLog && unlink(denied_in)==-1)
+		debuga(_("Cannot delete \"%s\": %s\n"),denied_in,strerror(errno));
 
 	return;
 }
