@@ -1,6 +1,6 @@
 /*
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
- *                                                            1998, 2011
+ *                                                            1998, 2012
  *
  * SARG donations:
  *      please look at http://sarg.sourceforge.net/donations.php
@@ -46,7 +46,7 @@ int geramail(const char *dirname, int debug, const char *outdir, const char *ema
 	char strip1[MAXLEN], strip2[MAXLEN], strip3[MAXLEN], strip4[MAXLEN], strip5[MAXLEN], strip6[MAXLEN], strip7[MAXLEN];
 	char *buf;
 	char warea[MAXLEN];
-	char Subject[100];
+	char Subject[120];
 	int totuser=0;
 	time_t t;
 	struct tm *local;
@@ -142,8 +142,8 @@ int geramail(const char *dirname, int debug, const char *outdir, const char *ema
 		exit(EXIT_FAILURE);
 	}
 
-	if (unlink(top2)) {
-		debuga(_("Cannot delete %s - %s\n"),top2,strerror(errno));
+	if (!KeepTempLog && unlink(top2)) {
+		debuga(_("Cannot delete \"%s\": %s\n"),top2,strerror(errno));
 		exit(EXIT_FAILURE);
 	}
 
@@ -253,8 +253,8 @@ int geramail(const char *dirname, int debug, const char *outdir, const char *ema
 #endif
 
 	fclose(fp_top1);
-	if (unlink(top1)) {
-		debuga(_("Cannot delete %s - %s\n"),top1,strerror(errno));
+	if (!KeepTempLog && unlink(top1)) {
+		debuga(_("Cannot delete \"%s\": %s\n"),top1,strerror(errno));
 		exit(EXIT_FAILURE);
 	}
 
@@ -273,8 +273,12 @@ int geramail(const char *dirname, int debug, const char *outdir, const char *ema
 		while(fgets(warea,sizeof(warea),fp_top3)!=NULL)
 			fputs(warea,stdout);
 	} else {
-		snprintf(Subject,sizeof(Subject),_("SARG report, %s"),asctime(local));
+		/* TRANSLATORS: The string is formatted using strftime. You can use
+		   any string formatting marker allowed by strftime. */
+		strftime(Subject,sizeof(Subject),_("SARG report, %c"),local);
 		snprintf(warea,sizeof(warea),"%s -s \"%s\" \"%s\" <\"%s\"",MailUtility,Subject,email,top3);
+		if (debug)
+			debuga(_("Sending mail with command: %s\n"),warea);
 		cstatus=system(warea);
 		if (!WIFEXITED(cstatus) || WEXITSTATUS(cstatus)) {
 			debuga(_("command return status %d\n"),WEXITSTATUS(cstatus));
