@@ -1,6 +1,6 @@
 /*
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
- *                                                            1998, 2011
+ *                                                            1998, 2012
  *
  * SARG donations:
  *      please look at http://sarg.sourceforge.net/donations.php
@@ -174,6 +174,7 @@ struct sort_list topsite_sort[]=
 	{"BYTES",TOPSITE_SORT_BYTES},
 	{"CONNECT",TOPSITE_SORT_CONNECT},
 	{"TIME",TOPSITE_SORT_TIME},
+	{"USER",TOPSITE_SORT_USER},
 };
 
 struct sort_list user_sort[]=
@@ -456,7 +457,9 @@ static void parmtest(char *buf)
 
 	if (getparam_quoted("title",buf,Title,sizeof(Title))>0) return;
 
-	if (getparam_bool("resolve_ip",buf,&Ip2Name)>0) return;
+	if (strncasecmp(buf,"resolve_ip",10)==0) {
+		if (ip2name_config(buf+10)>0) return;
+	}
 
 	if (getparam_bool("user_ip",buf,&UserIp)>0) return;
 
@@ -674,7 +677,14 @@ static void parmtest(char *buf)
 		return;
 	}
 
-	if (getparam_bool("graphs",buf,&Graphs)>0) return;
+	if (getparam_bool("graphs",buf,&Graphs)>0) {
+#ifndef HAVE_GD
+		if (Graphs)
+			debugaz(_("No graphs available as sarg was not compiled with libgd. Set \"graphs\" to \"no\" in %s to disable this warning\n"),
+				ConfigFile);
+#endif
+		return;
+	}
 
 	if (getparam_string("graph_days_bytes_bar_color",buf,GraphDaysBytesBarColor,sizeof(GraphDaysBytesBarColor))>0) return;
 
@@ -739,6 +749,8 @@ static void parmtest(char *buf)
 	if (getparam_string("sorttable",buf,SortTableJs,sizeof(SortTableJs))>0) return;
 
 	if (getparam_string("hostalias",buf,HostAliasFile,sizeof(HostAliasFile))>0) return;
+
+	if (getparam_bool("keep_temp_log",buf,&KeepTempLog)>0) return;
 	
 	if(strstr(buf,"squid24") != 0) {
 		squid24=true;
