@@ -1,6 +1,6 @@
 /*
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
- *                                                            1998, 2011
+ *                                                            1998, 2012
  *
  * SARG donations:
  *      please look at http://sarg.sourceforge.net/donations.php
@@ -633,6 +633,17 @@ static void greport_plot(const struct userinfostruct *uinfo,struct PlotStruct *p
 void greport_prepare(void)
 {
 #ifdef HAVE_GD
+	if (!Graphs) {
+		if (debugz)
+			debugaz(_("Graphs disabled as requested in %s\n"),ConfigFile);
+		return;
+	}
+	if (GraphFont[0]=='\0') {
+		if (debugz)
+			debugaz(_("Graphs disabled as no font names were provided in %s\n"),ConfigFile);
+		return;
+	}
+
 	if(access(GraphFont, R_OK) != 0) {
 		debuga(_("(grepday) Fontname %s not found\n"),GraphFont);
 		exit(EXIT_FAILURE);
@@ -672,7 +683,8 @@ void greport_day(const struct userinfostruct *uinfo)
 		return;
 	}
 	if(!Graphs || GraphFont[0]=='\0') {
-		unlink(wdirname);
+		if (!KeepTempLog && unlink(wdirname))
+			debuga(_("Cannot delete \"%s\": %s\n"),wdirname,strerror(errno));
 		return;
 	}
 
@@ -713,8 +725,8 @@ void greport_day(const struct userinfostruct *uinfo)
 		}
 	}
 	fclose(fp_in);
-	if (unlink(wdirname)) {
-		debuga(_("Cannot delete %s - %s\n"),wdirname,strerror(errno));
+	if (!KeepTempLog && unlink(wdirname)) {
+		debuga(_("Cannot delete \"%s\": %s\n"),wdirname,strerror(errno));
 		exit(EXIT_FAILURE);
 	}
 
