@@ -1,6 +1,6 @@
 /*
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
- *                                                            1998, 2011
+ *                                                            1998, 2012
  *
  * SARG donations:
  *      please look at http://sarg.sourceforge.net/donations.php
@@ -51,7 +51,7 @@ void htmlrel(void)
 	double perc=0, perc2=0, ouperc=0, inperc=0;
 	int count;
 	int cstatus;
-	int have_denied_report;
+	bool have_denied_report;
 	const char *sort_field;
 	const char *sort_order;
 	char siteind[MAX_TRUNCATED_URL];
@@ -60,13 +60,13 @@ void htmlrel(void)
 	const struct userinfostruct *uinfo;
 	userscan uscan;
 
-	if (snprintf(tmp2,sizeof(tmp2),"%s/sargtmp.unsort",tmp)>=sizeof(tmp2)) {
-		debuga(_("File name too long: %s/sargtmp.unsort\n"),tmp);
+	if (snprintf(tmp2,sizeof(tmp2),"%s/sargtmp.int_unsort",tmp)>=sizeof(tmp2)) {
+		debuga(_("File name too long: %s/sargtmp.int_unsort\n"),tmp);
 		exit(EXIT_FAILURE);
 	}
 
-	if (snprintf(tmp3,sizeof(tmp3),"%s/sargtmp.log",tmp)>=sizeof(tmp3)) {
-		debuga(_("File name too long: %s/sargtmp.log\n"),tmp);
+	if (snprintf(tmp3,sizeof(tmp3),"%s/sargtmp.int_log",tmp)>=sizeof(tmp3)) {
+		debuga(_("File name too long: %s/sargtmp.int_log\n"),tmp);
 		exit(EXIT_FAILURE);
 	}
 
@@ -75,8 +75,6 @@ void htmlrel(void)
 	totelap=globstat.elap;
 	ntotuser=globstat.totuser;
 
-	greport_prepare();
-
 	sort_labels(&sort_field,&sort_order);
 
 	uscan=userinfo_startscan();
@@ -103,9 +101,6 @@ void htmlrel(void)
 		}
 		tmpsort(uinfo);
 
-		report_day(uinfo);
-		greport_day(uinfo);
-
 		if (snprintf(arqin,sizeof(arqin),"%s/htmlrel.txt",tmp)>=sizeof(arqin)) {
 			debuga(_("Input file name too long: %s/htmlrel.txt\n"),tmp);
 			exit(EXIT_FAILURE);
@@ -120,14 +115,14 @@ void htmlrel(void)
 			debuga(_("Output file name too long: %s/%s/%s.html\n"),outdirname,uinfo->filename,uinfo->filename);
 			exit(EXIT_FAILURE);
 		}
-		if (snprintf(duser,sizeof(duser),"%s/%s/denied_%s.html",tmp,uinfo->filename,uinfo->filename)>=sizeof(duser)) {
-			debuga(_("File name too long: %s/%s/denied_%s.html\n"),tmp,uinfo->filename,uinfo->filename);
+		if (snprintf(duser,sizeof(duser),"%s/denied_%s.html",outdirname,uinfo->filename)>=sizeof(duser)) {
+			debuga(_("File name too long: %s/denied_%s.html\n"),outdirname,uinfo->filename);
 			exit(EXIT_FAILURE);
 		}
 		if(access(duser, R_OK) != 0)
-			have_denied_report=0;
+			have_denied_report=false;
 		else
-			have_denied_report=1;
+			have_denied_report=true;
 
 		if ((line=longline_create())==NULL) {
 			debuga(_("Not enough memory to read file %s\n"),arqin);
@@ -192,17 +187,16 @@ void htmlrel(void)
 		fprintf(fp_ou,"<tr><th class=\"header_c\">%s</th></tr>\n",_("User report"));
 		close_html_header(fp_ou);
 
+		if (have_denied_report) {
+			fputs("<div class=\"report\"><table cellpadding=\"1\" cellspacing=\"2\">\n",fp_ou);
+			fprintf(fp_ou,"<tr><td class=\"header_l\" colspan=\"11\"><a href=\"denied_%s.html\">%s</a></td></tr>\n",uinfo->filename,_("SmartFilter report"));
+			fputs("<tr><td></td></tr>\n</table></div>\n",fp_ou);
+		}
+
 		fputs("<div class=\"report\"><table cellpadding=\"2\" cellspacing=\"1\"",fp_ou);
 		if (SortTableJs[0]) fputs(" class=\"sortable\"",fp_ou);
 		fputs(">\n",fp_ou);
 
-		/*! \bug This line lays before the header of the table and is therefore invalid
-		HTML. It will also break the sorttable script. Beside, I suspect it serves no purpose.
-		*/
-		if(have_denied_report) {
-			fprintf(fp_ou,"<tr><td class=\"header_l\" colspan=\"11\"><a href=\"denied_%s.html\">%s</a> %s</td></tr>\n",uinfo->filename,_("SmartFilter"),_("Report"));
-		}
-
 		fputs("<thead><tr><th class=\"sorttable_nosort\"></th><th class=\"header_l",fp_ou);
 		if (SortTableJs[0]) fputs(" sorttable_alpha",fp_ou);
 		fprintf(fp_ou,"\">%s</th>",_("ACCESSED SITE"));
@@ -330,9 +324,9 @@ void htmlrel(void)
 				count++;
 			} else if ((ReportType & REPORT_TYPE_SITE_USER_TIME_DATE) != 0) {
 				url_to_file(url,siteind,sizeof(siteind));
-				snprintf(tmp2,sizeof(tmp2),"%s/%s/tt%s-%s.html",outdirname,uinfo->filename,uinfo->filename,siteind);
-				if (unlink(tmp2)!=0) {
-					debuga(_("Cannot delete unused file \"%s\" - %s\n"),tmp2,strerror(errno));
+				snprintf(warea,sizeof(warea),"%s/%s/tt%s-%s.html",outdirname,uinfo->filename,uinfo->filename,siteind);
+				if (unlink(warea)!=0) {
+					debuga(_("Cannot delete \"%s\": %s\n"),warea,strerror(errno));
 				}
 			}
 
@@ -406,6 +400,11 @@ void htmlrel(void)
 					exit(EXIT_FAILURE);
 				}
 
+				if (unlink(tmp2)) {
+					debuga(_("Cannot delete \"%s\": %s\n"),tmp2,strerror(errno));
+					exit(EXIT_FAILURE);
+				}
+
 				olduserip[0]='\0';
 
 				if ((line1=longline_create())==NULL) {
@@ -456,12 +455,8 @@ void htmlrel(void)
 				fclose(fp_ip);
 				longline_destroy(&line1);
 
-				if (unlink(tmp2)) {
-					debuga(_("Cannot delete %s - %s\n"),tmp2,strerror(errno));
-					exit(EXIT_FAILURE);
-				}
 				if (unlink(tmp3)) {
-					debuga(_("Cannot delete %s - %s\n"),tmp3,strerror(errno));
+					debuga(_("Cannot delete \"%s\": %s\n"),tmp3,strerror(errno));
 					exit(EXIT_FAILURE);
 				}
 
@@ -491,13 +486,13 @@ void htmlrel(void)
 		longline_destroy(&line);
 
 		if (iprel && arqip[0]) {
-			if (unlink(arqip)) {
-				debuga(_("Cannot delete %s - %s\n"),arqip,strerror(errno));
+			if (!KeepTempLog && unlink(arqip)) {
+				debuga(_("Cannot delete \"%s\": %s\n"),arqip,strerror(errno));
 				exit(EXIT_FAILURE);
 			}
 		}
-		if (unlink(arqin)) {
-			debuga(_("Cannot delete %s - %s\n"),arqin,strerror(errno));
+		if (!KeepTempLog && unlink(arqin)) {
+			debuga(_("Cannot delete \"%s\": %s\n"),arqin,strerror(errno));
 			exit(EXIT_FAILURE);
 		}
 
@@ -596,7 +591,6 @@ void htmlrel(void)
 	}
 
 	userinfo_stopscan(uscan);
-	greport_cleanup();
 
 	return;
 }
