@@ -55,6 +55,12 @@ struct userinfostruct
 	int topuser;
 	//! A general purpose flag that can be set when scanning the user's list.
 	int flag;
+#ifdef ENABLE_DOUBLE_CHECK_DATA
+	//! Total number of bytes.
+	long long int nbytes;
+	//! Total time spent processing the requests.
+	long long int elap;
+#endif
 };
 
 //! Scan through the known users.
@@ -78,6 +84,9 @@ struct globalstatstruct
 	int totuser;
 };
 
+//! The object to store the daily statistics.
+typedef struct DayStruct *DayObject;
+
 // auth.c
 void htaccess(const struct userinfostruct *uinfo);
 
@@ -141,7 +150,10 @@ void htmlrel(void);
 void index_only(const char *dirname,int debug);
 
 // ip2name.c
+int ip2name_config(const char *param);
+void ip2name_forcedns(void);
 void ip2name(char *ip,int ip_len);
+void ip2name_cleanup(void);
 void name2ip(char *name,int name_size);
 
 // lastlog.c
@@ -178,7 +190,7 @@ void siteuser(void);
 void smartfilter_report(void);
 
 // sort.c
-void sort_users_log(const char *tmp, int debug);
+void sort_users_log(const char *tmp, int debug,struct userinfostruct *uinfo);
 void tmpsort(const struct userinfostruct *uinfo);
 void sort_labels(const char **label,const char **order);
 
@@ -192,11 +204,16 @@ void topsites(void);
 void topuser(void);
 
 // totday.c
-void day_totalize(const char *tmp, const struct userinfostruct *uinfo);
+DayObject day_prepare(void);
+void day_cleanup(DayObject ddata);
+void day_newuser(DayObject ddata);
+void day_addpoint(DayObject ddata,const char *date, const char *time, long long int elap, long long int bytes);
+void day_totalize(DayObject ddata,const char *tmp, const struct userinfostruct *uinfo);
 
 // url.c
 void read_hostalias(const char *Filename);
 void free_hostalias(void);
+const char *skip_scheme(const char *url);
 const char *process_url(char *url,bool full_url);
 void url_hostname(const char *url,char *hostname,int hostsize);
 
@@ -286,5 +303,6 @@ void removetmp(const char *outdir);
 void zdate(char *ftime,int ftimesize, const char *DateFormat);
 char *get_param_value(const char *param,char *line);
 int compar( const void *, const void * );
-void unlinkdir(const char *dir,int contentonly);
+void unlinkdir(const char *dir,bool contentonly);
+void emptytmpdir(const char *dir);
 int extract_address_mask(const char *buf,const char **text,unsigned char *ipv4,unsigned short int *ipv6,int *nbits,const char **next);
