@@ -1,6 +1,6 @@
 /*
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
- *                                                            1998, 2011
+ *                                                            1998, 2012
  *
  * SARG donations:
  *      please look at http://sarg.sourceforge.net/donations.php
@@ -44,7 +44,7 @@ void make_index(void)
 		sprintf(wdir,"%sindex.html",outdir);
 		if(access(wdir, R_OK) == 0) {
 			if (unlink(wdir)) {
-				debuga(_("Cannot delete %s - %s\n"),wdir,strerror(errno));
+				debuga(_("Cannot delete \"%s\": %s\n"),wdir,strerror(errno));
 				exit(EXIT_FAILURE);
 			}
 		}
