@@ -1,6 +1,6 @@
 /*
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
- *                                                            1998, 2011
+ *                                                            1998, 2012
  *
  * SARG donations:
  *      please look at http://sarg.sourceforge.net/donations.php
@@ -47,7 +47,7 @@ void index_only(const char *dirname,int debug)
 			continue;
 		}
 		if (unlink(remove) == -1) {
-			debuga(_("Failed to remove the file %s: %s\n"),remove,strerror(errno));
+			debuga(_("Cannot delete \"%s\": %s\n"),remove,strerror(errno));
 		}
 	}
 
