@@ -1,6 +1,6 @@
 /*
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
- *                                                            1998, 2011
+ *                                                            1998, 2012
  *
  * SARG donations:
  *      please look at http://sarg.sourceforge.net/donations.php
@@ -84,7 +84,7 @@ void mklastlog(const char *outdir)
 	}
 
 	if (unlink(temp)) {
-		debuga(_("Cannot delete %s - %s\n"),temp,strerror(errno));
+		debuga(_("Cannot delete \"%s\": %s\n"),temp,strerror(errno));
 		exit(EXIT_FAILURE);
 	}
 
@@ -92,7 +92,7 @@ void mklastlog(const char *outdir)
 		snprintf(temp,sizeof(temp),"%slastlog",outdir);
 		if(access(temp, R_OK) == 0) {
 			if (unlink(temp)) {
-				debuga(_("Cannot delete %s - %s\n"),temp,strerror(errno));
+				debuga(_("Cannot delete \"%s\": %s\n"),temp,strerror(errno));
 				exit(EXIT_FAILURE);
 			}
 		}
@@ -128,7 +128,7 @@ void mklastlog(const char *outdir)
 	fclose(fp_in);
 	snprintf(temp,sizeof(temp),"%slastlog",outdir);
 	if (unlink(temp) == -1) {
-		debuga(_("Failed to delete the file %s\n"),temp);
+		debuga(_("Cannot delete \"%s\": %s\n"),temp,strerror(errno));
 	}
 
 	return;
