@@ -1,6 +1,6 @@
 /*
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
- *                                                            1998, 2011
+ *                                                            1998, 2012
  *
  * SARG donations:
  *      please look at http://sarg.sourceforge.net/donations.php
@@ -108,7 +108,6 @@ int main(int argc,char *argv[])
 	char *linebuf;
 	const char *url;
 	char *full_url;
-	char *urly;
 	char user[MAX_USER_LEN];
 	char splitprefix[MAXLEN];
 	enum InputLogFormat ilf;
@@ -126,6 +125,8 @@ int main(int argc,char *argv[])
 	int  iarq=0;
 	int isa_ncols=0,isa_cols[ISACOL_Last];
 	int lastlog=-1;
+	long int nbytes;
+	long int elap_time;
 	bool from_stdin;
 	bool from_pipe;
 	int blen;
@@ -144,30 +145,46 @@ int main(int argc,char *argv[])
 	struct tm *t;
 	unsigned long recs1=0UL;
 	unsigned long recs2=0UL;
+	unsigned long int lines_read=0UL;
+	unsigned long int records_kept=0UL;
+	unsigned long int nusers=0UL;
 	int OutputNonZero = REPORT_EVERY_X_LINES ;
 	bool download_flag=false;
-	char *download_url=NULL;
+	char download_url[MAXLEN];
 	struct getwordstruct gwarea;
 	longline line;
 	time_t tnum;
+	time_t start_time;
+	time_t end_time;
+	time_t read_start_time;
+	time_t read_end_time;
+	time_t process_start_time;
+	time_t process_end_time;
+	double read_elapsed;
+	double process_elapsed;
 	struct stat logstat;
 	struct userinfostruct *uinfo;
 	struct userfilestruct *first_user_file, *ufile, *ufile1, *prev_ufile;
 	static int split=0;
 	static int convert=0;
 	static int output_css=0;
+	static int show_statis=0;
 	int option_index;
 	static struct option long_options[]=
 	{
 		{"convert",no_argument,&convert,1},
 		{"css",no_argument,&output_css,1},
+		{"help",no_argument,NULL,'h'},
 		{"lastlog",required_argument,NULL,2},
 		{"keeplogs",no_argument,NULL,3},
 		{"split",no_argument,&split,1},
 		{"splitprefix",required_argument,NULL,'P'},
+		{"statistics",no_argument,&show_statis,1},
 		{0,0,0,0}
 	};
 
+	start_time=time(NULL);
+
 #ifdef HAVE_LOCALE_H
 	setlocale(LC_TIME,"");
 #endif
@@ -346,6 +363,7 @@ int main(int argc,char *argv[])
 	squid24=false;
 	dfrom=0;
 	duntil=0;
+	KeepTempLog=false;
 
 	bzero(IncludeUsers, sizeof(IncludeUsers));
 	bzero(ExcludeString, sizeof(ExcludeString));
@@ -360,7 +378,7 @@ int main(int argc,char *argv[])
 
 	strcpy(Title,_("Squid User Access Report"));
 
-	while((ch = getopt_long_only(argc, argv, "a:b:c:d:e:f:g:u:l:L:o:s:t:w:P:hijmnprvxyz",long_options,&option_index)) != -1){
+	while((ch = getopt_long_only(argc, argv, "a:b:c:d:e:f:g:hikl:L:mno:P:prs:t:u:vw:xyz",long_options,&option_index)) != -1){
 		switch(ch)
 		{
 			case 0:
@@ -374,7 +392,7 @@ int main(int argc,char *argv[])
 			case 'a':
 				safe_strcpy(addr,optarg,sizeof(addr));
 				break;
-			case 'b':
+			case 'b': //unused option
 				safe_strcpy(uagent,optarg,sizeof(uagent));
 				break;
 			case 'c':
@@ -399,6 +417,9 @@ int main(int argc,char *argv[])
 			case 'i':
 				iprel=true;
 				break;
+			case 'k':
+				KeepTempLog=true;
+				break;
 			case 'l':
 				if (NAccessLog>=MAXLOGS) {
 					debuga(_("Too many log files passed on command line with option -l.\n"));
@@ -452,7 +473,7 @@ int main(int argc,char *argv[])
 
 				if(strstr(optarg,"-") == 0) {
 					if(sscanf(optarg,"%d:%d",&h1,&m1)!=2) {
-						debuga(_("Time period passed on the command line with option -t must be MM:SS\n"));
+						debuga(_("Time period passed on the command line with option -t must be HH:MM\n"));
 						exit(EXIT_FAILURE);
 					}
 					hm=h1*100+m1;
@@ -460,7 +481,7 @@ int main(int argc,char *argv[])
 					snprintf(hm_str,sizeof(hm_str),"%02d:%02d",h1,m1);
 				} else {
 					if(sscanf(optarg,"%d:%d-%d:%d",&h1,&m1,&h2,&m2)!=4) {
-						debuga(_("Time range passed on the command line with option -t must be MM:SS-MM:SS\n"));
+						debuga(_("Time range passed on the command line with option -t must be HH:MM-HH:MM\n"));
 						exit(EXIT_FAILURE);
 					}
 					hm=h1*100+m1;
@@ -481,7 +502,7 @@ int main(int argc,char *argv[])
 			case 'x':
 				debug++;
 				break;
-			case 'y':
+			case 'y': //unused option
 				langcode++;
 				break;
 			case 'z':
@@ -532,7 +553,7 @@ int main(int argc,char *argv[])
 
 	if(userip) UserIp=true;
 
-	if(dns) Ip2Name=true;
+	if(dns) ip2name_forcedns();
 
 	if (lastlog>=0) LastLog=lastlog;
 
@@ -644,12 +665,13 @@ int main(int argc,char *argv[])
 	}
 
 	if(access(tmp, R_OK) == 0) {
-		unlinkdir(tmp,1);
+		if (debug) debuga(_("Deleting temporary directory \"%s\"\n"),tmp);
+		emptytmpdir(tmp);
 	}
 	my_mkdir(tmp);
-	snprintf(denied_unsort,sizeof(denied_unsort),"%s/denied.log.unsort",tmp);
-	snprintf(denied_sort,sizeof(denied_sort),"%s/denied.log",tmp);
-	snprintf(authfail_unsort,sizeof(authfail_unsort),"%s/authfail.log.unsort",tmp);
+	snprintf(denied_unsort,sizeof(denied_unsort),"%s/denied.int_unsort",tmp);
+	snprintf(denied_sort,sizeof(denied_sort),"%s/denied.int_log",tmp);
+	snprintf(authfail_unsort,sizeof(authfail_unsort),"%s/authfail.int_unsort",tmp);
 
 	if(debug) {
 		debuga(_("Parameters:\n"));
@@ -666,6 +688,7 @@ int main(int argc,char *argv[])
 		if(strcmp(df,"w") == 0)
 			debuga(_("                     Date format (-g) = Sites & Users (yyyy/ww)\n"));
 		debuga(_("                       IP report (-i) = %s\n"),(iprel) ? _("Yes") : _("No"));
+		debuga(_("            Keep temporary files (-k) = %s\n"),(KeepTempLog) ? _("Yes") : _("No"));
 		for (iarq=0 ; iarq<NAccessLog ; iarq++)
 			debuga(_("                       Input log (-l) = %s\n"),AccessLog[iarq]);
 		for (iarq=0 ; iarq<NRedirectorLogs ; iarq++)
@@ -698,6 +721,7 @@ int main(int argc,char *argv[])
 		if(strcmp(df,"w") == 0)
 			printf(_("                     Date format (-g) = Sites & Users (yyyy/ww)\n"));
 		printf(_("                       IP report (-i) = %s\n"),(iprel) ? _("Yes") : _("No"));
+		printf(_("            Keep temporary files (-k) = %s\n"),(KeepTempLog) ? _("Yes") : _("No"));
 		for (iarq=0 ; iarq<NAccessLog ; iarq++)
 			printf(_("                       Input log (-l) = %s\n"),AccessLog[iarq]);
 		for (iarq=0 ; iarq<NRedirectorLogs ; iarq++)
@@ -764,7 +788,7 @@ int main(int argc,char *argv[])
 		exit(EXIT_FAILURE);
 	}
 
-	snprintf(sz_Download_Unsort,sizeof(sz_Download_Unsort),"%s/download.unsort", tmp);
+	snprintf(sz_Download_Unsort,sizeof(sz_Download_Unsort),"%s/download.int_unsort", tmp);
 
 	if(DataFile[0]=='\0') {
 		if((ReportType & REPORT_TYPE_DENIED) != 0) {
@@ -782,6 +806,7 @@ int main(int argc,char *argv[])
 		}
 	}
 
+	read_start_time=time(NULL);
 	for (iarq=0 ; iarq<NAccessLog ; iarq++) {
 		strcpy(arq,AccessLog[iarq]);
 
@@ -846,6 +871,7 @@ int main(int argc,char *argv[])
 
 		while ((linebuf=longline_read(fp_in,line))!=NULL) {
 			blen=strlen(linebuf);
+			lines_read++;
 
 			if (ilf==ILF_Unknown) {
 				if(strncmp(linebuf,"#Software: Mic",14) == 0) {
@@ -1322,8 +1348,6 @@ int main(int argc,char *argv[])
 			for (str=code ; *str ; str++)
 				if (*str=='\t') *str=' ';
 
-			urly=full_url;
-
 			if(ilf!=ILF_Sarg) {
 				/*
 				The full URL is not saved in sarg log. There is no point in testing the URL to detect
@@ -1331,7 +1355,7 @@ int main(int argc,char *argv[])
 				*/
 				download_flag=is_download_suffix(full_url);
 				if (download_flag) {
-					download_url=full_url;
+					safe_strcpy(download_url,full_url,sizeof(download_url));
 					download_count++;
 				}
 			} else
@@ -1377,11 +1401,7 @@ int main(int argc,char *argv[])
 				} else {
 					strlow(user);
 					if(NtlmUserFormat == NTLMUSERFORMAT_USER) {
-						if((str = strchr(user,'_')) != 0) {
-							strcpy(warea,str+1);
-							strcpy(user,warea);
-						}
-						if((str = strchr(user,'+')) != 0) {
+						if ((str=strchr(user,'+'))!=NULL || (str=strchr(user,'\\'))!=NULL || (str=strchr(user,'_'))!=NULL) {
 							strcpy(warea,str+1);
 							strcpy(user,warea);
 						}
@@ -1410,10 +1430,14 @@ int main(int argc,char *argv[])
 			if(strcmp(user,"-") ==0 || strcmp(user," ") ==0 || strcmp(user,"") ==0 || strcmp(user,":") ==0)
 				continue;
 
+			nbytes=atol(tam);
+			if (nbytes<0) nbytes=0;
+
+			elap_time=atol(elap);
+			if (elap_time<0) elap_time=0;
 			if(max_elapsed) {
-				if(atol(elap)>max_elapsed) {
-					elap[0]='0';
-					elap[1]='\0';
+				if(elap_time>max_elapsed) {
+					elap_time=0;
 				}
 			}
 
@@ -1440,6 +1464,7 @@ int main(int argc,char *argv[])
 				uinfo=userinfo_create(user);
 				ufile->user=uinfo;
 				uinfo->id_is_ip=id_is_ip;
+				nusers++;
 			} else {
 				if (prev_ufile) {
 					prev_ufile->next=ufile->next;
@@ -1447,6 +1472,10 @@ int main(int argc,char *argv[])
 					first_user_file=ufile;
 				}
 			}
+#ifdef ENABLE_DOUBLE_CHECK_DATA
+			ufile->user->nbytes+=nbytes;
+			ufile->user->elap+=elap_time;
+#endif
 
 			if (ufile->file==NULL) {
 				if (nopen>=maxopenfiles) {
@@ -1464,8 +1493,8 @@ int main(int argc,char *argv[])
 						}
 					}
 				}
-				if (snprintf (tmp3, sizeof(tmp3), "%s/%s.unsort", tmp, ufile->user->filename)>=sizeof(tmp3)) {
-					debuga(_("Temporary user file name too long: %s/%s.unsort\n"), tmp, ufile->user->filename);
+				if (snprintf (tmp3, sizeof(tmp3), "%s/%s.user_unsort", tmp, ufile->user->filename)>=sizeof(tmp3)) {
+					debuga(_("Temporary user file name too long: %s/%s.user_unsort\n"), tmp, ufile->user->filename);
 					exit(EXIT_FAILURE);
 				}
 				if ((ufile->file = MY_FOPEN (tmp3, "a")) == NULL) {
@@ -1474,28 +1503,18 @@ int main(int argc,char *argv[])
 				}
 			}
 
-			/*if ( strcmp ( user , sz_Last_User ) != 0 ) {
-				if ( fp_Write_User )
-					fclose( fp_Write_User ) ;
-				sprintf (tmp3, "%s/%s.unsort", tmp, user);
-
-				if ((fp_Write_User = MY_FOPEN (tmp3, "a")) == NULL) {
-					fprintf (stderr, "%s: (log) %s: %s - %s\n", argv[0], _("Cannot open temporary file"), tmp3, strerror(errno));
-					exit (1);
-				}
-				strcpy( sz_Last_User , user ) ;
-			}*/
-			if (fprintf(ufile->file, "%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\n",dia,hora,ip,url,tam,code,elap,smartfilter)<=0) {
+			if (fprintf(ufile->file, "%s\t%s\t%s\t%s\t%ld\t%s\t%ld\t%s\n",dia,hora,ip,url,nbytes,code,elap_time,smartfilter)<=0) {
 				debuga(_("Write error in the log file of user %s\n"),user);
 				exit(EXIT_FAILURE);
 			}
+			records_kept++;
 
 			if(fp_log && ilf!=ILF_Sarg)
-				fprintf(fp_log, "%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\n",dia,hora,user,ip,url,tam,code,elap,smartfilter);
+				fprintf(fp_log, "%s\t%s\t%s\t%s\t%s\t%ld\t%s\t%ld\t%s\n",dia,hora,user,ip,url,nbytes,code,elap_time,smartfilter);
 
 			totregsg++;
 
-			if(!dataonly && download_flag && download_url && strstr(code,"DENIED") == 0) {
+			if(!dataonly && download_flag && strstr(code,"DENIED") == 0) {
 				ndownload = 1;
 
 				if ( ! fp_Download_Unsort ) {
@@ -1509,13 +1528,13 @@ int main(int argc,char *argv[])
 
 			if((ReportType & REPORT_TYPE_DENIED) != 0) {
 				if(fp_denied && strstr(code,"DENIED/403") != 0) {
-					fprintf(fp_denied, "%s\t%s\t%s\t%s\t%s\n",dia,hora,user,ip,urly);
+					fprintf(fp_denied, "%s\t%s\t%s\t%s\t%s\n",dia,hora,user,ip,full_url);
 					denied_count++;
 				}
 			}
 			if((ReportType & REPORT_TYPE_AUTH_FAILURES) != 0) {
 				if(fp_authfail && (strstr(code,"DENIED/401") != 0 || strstr(code,"DENIED/407") != 0)) {
-					fprintf(fp_authfail, "%s\t%s\t%s\t%s\t%s\n",dia,hora,user,ip,urly);
+					fprintf(fp_authfail, "%s\t%s\t%s\t%s\t%s\n",dia,hora,user,ip,full_url);
 					authfail_count++;
 				}
 			}
@@ -1536,13 +1555,13 @@ int main(int argc,char *argv[])
 			if(debugm){
 				printf("IP=\t%s\n",ip);
 				printf("USER=\t%s\n",user);
-				printf("ELAP=\t%s\n",elap);
+				printf("ELAP=\t%ld\n",elap_time);
 				printf("DATE=\t%s\n",dia);
 				printf("TIME=\t%s\n",hora);
 				printf("FUNC=\t%s\n",fun);
 				printf("URL=\t%s\n",url);
 				printf("CODE=\t%s\n",code);
-				printf("LEN=\t%s\n",tam);
+				printf("LEN=\t%ld\n",nbytes);
 			}
 		}
 		if (!from_stdin) {
@@ -1555,6 +1574,8 @@ int main(int argc,char *argv[])
 			}
 		}
 	}
+	read_end_time=time(NULL);
+	read_elapsed=(double)read_end_time-(double)read_start_time;
 
 	if (debug)
 		debuga(_("   Records read: %ld, written: %ld, excluded: %ld\n"),totregsl,totregsg,totregsx);
@@ -1635,7 +1656,7 @@ int main(int argc,char *argv[])
 		char end_hour[128];
 		char val2[40];
 		char val4[255];//val4 must not be bigger than arq_log without fixing the strcpy below
-		
+
 		fclose(fp_log);
 		safe_strcpy(end_hour,tbuf2,sizeof(end_hour));
 		strftime(val2,sizeof(val2),"%d%m%Y",&period.start);
@@ -1681,33 +1702,50 @@ int main(int argc,char *argv[])
 			debuga(_("sort command: %s\n"),csort);
 			exit(EXIT_FAILURE);
 		}
-		if (unlink(denied_unsort)) {
-			debuga(_("Cannot delete %s - %s\n"),denied_unsort,strerror(errno));
+		if (!KeepTempLog && unlink(denied_unsort)) {
+			debuga(_("Cannot delete \"%s\": %s\n"),denied_unsort,strerror(errno));
 			exit(EXIT_FAILURE);
 		}
 	}
 
-	sort_users_log(tmp, debug);
-
+	process_start_time=time(NULL);
 	if(DataFile[0] != '\0')
 		data_file(tmp);
 	else
 		gerarel();
+	process_end_time=time(NULL);
+	process_elapsed=(double)process_end_time-(double)process_start_time;
 
 	if((ReportType & REPORT_TYPE_DENIED) != 0) {
-		unlink(denied_sort);
+		if (!KeepTempLog && unlink(denied_sort) && errno!=ENOENT)
+			debuga(_("Cannot delete \"%s\": %s\n"),denied_sort,strerror(errno));
 	}
 
-	if(strcmp(tmp,"/tmp") != 0) {
+	if(!KeepTempLog && strcmp(tmp,"/tmp") != 0) {
 		unlinkdir(tmp,0);
 	}
 
+	ip2name_cleanup();
 	free_hostalias();
 	userinfo_free();
 	if(userfile)
 		free(userfile);
 	close_usertab();
 
+	end_time=time(NULL);
+
+	if (show_statis) {
+		double elapsed=(double)end_time-(double)start_time;
+		debuga(_("Total execution time: %.0lf seconds\n"),elapsed);
+		if (read_elapsed>0.) {
+			debuga(_("Lines read: %lu lines in %.0lf seconds (%.0lf lines/s)\n"),lines_read,read_elapsed,(double)lines_read/read_elapsed);
+		}
+		if (process_elapsed>0.) {
+			debuga(_("Processed records: %lu records in %.0lf seconds (%.0lf records/s)\n"),records_kept,process_elapsed,(double)records_kept/process_elapsed);
+			debuga(_("Users: %lu users in %.0lf seconds (%.0lf users/s)\n"),nusers,process_elapsed,(double)nusers/process_elapsed);
+		}
+	}
+
 	if(debug)
 		debuga(_("End\n"));
 
