@@ -1,6 +1,6 @@
 /*
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
- *                                                            1998, 2011
+ *                                                            1998, 2012
  *
  * SARG donations:
  *      please look at http://sarg.sourceforge.net/donations.php
@@ -104,8 +104,8 @@ static void getlog(void)
 		debuga(_("sort command: %s\n"),cmd);
 		exit(EXIT_FAILURE);
 	}
-	if (unlink(template1)) {
-		debuga(_("Cannot delete %s - %s\n"),template1,strerror(errno));
+	if (!KeepTempLog && unlink(template1)) {
+		debuga(_("Cannot delete \"%s\": %s\n"),template1,strerror(errno));
 		exit(EXIT_FAILURE);
 	}
 	datashow(template2);
@@ -288,8 +288,8 @@ static void datashow(const char *tmp)
 
 	puts("</table>\n</div>\n</body>\n</html>\n");
 	fclose(fin);
-	if (unlink(tmp)) {
-		debuga(_("Cannot delete %s - %s\n"),tmp,strerror(errno));
+	if (!KeepTempLog && unlink(tmp)) {
+		debuga(_("Cannot delete \"%s\": %s\n"),tmp,strerror(errno));
 		exit(EXIT_FAILURE);
 	}
 	fflush(NULL);
