@@ -1,6 +1,6 @@
 /*
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
- *                                                            1998, 2011
+ *                                                            1998, 2012
  *
  * SARG donations:
  *      please look at http://sarg.sourceforge.net/donations.php
@@ -140,8 +140,11 @@ void report_day(const struct userinfostruct *uinfo)
 		label=_("BYTES");
 		fputs("<table class=\"report\" cellpadding=\"0\" cellspacing=\"2\">\n", fp_ou);
 		fputs("<tr><th class=\"header_c\"></th>",fp_ou);
-		for( i = 0; i < hours.len; i++ )
-			fprintf(fp_ou, "<td class=\"header_c\">%02d%s<br>%s</td>\n", hours.list[ i ], _("H"), label );
+		for( i = 0; i < hours.len; i++ ) {
+			/* TRANSLATORS: It is an hour in the hourly report */
+			sprintf(data,_("%02dH"),hours.list[ i ]);
+			fprintf(fp_ou, "<td class=\"header_c\">%s<br>%s</td>\n", data, label );
+		}
 		fprintf(fp_ou, "<td class=\"header_c\">%s<br>%s</td></tr>\n", _("TOTAL"), label );
 
 		memset(tttime,0,sizeof(tttime));
@@ -177,14 +180,23 @@ void report_day(const struct userinfostruct *uinfo)
 		}
 		fprintf(fp_ou, "<td class=\"header_r\">%s</td></tr>\n",fixnum(tt,1));
 		fputs("</table>\n",fp_ou);
+#ifdef ENABLE_DOUBLE_CHECK_DATA
+		if (tt!=uinfo->nbytes) {
+			debuga(_("Total downloaded bytes is %"PRIi64" instead of %"PRIi64" in the hourly report of user %s\n"),
+						 (int64_t)tt,(int64_t)uinfo->nbytes,uinfo->label);
+			exit(EXIT_FAILURE);
+		}
+#endif
 	}
 
 	if((datetimeby & DATETIME_ELAP)!=0) {
 		label=_("H:M:S");
 		fputs("<table class=\"report\" cellpadding=\"0\" cellspacing=\"2\">\n", fp_ou);
 		fputs("<tr><th class=\"header_c\"></th>",fp_ou);
-		for( i = 0; i < hours.len; i++ )
-			fprintf(fp_ou, "<td class=\"header_c\">%02d%s<br>%s</td>\n", hours.list[ i ], _("H"), label );
+		for( i = 0; i < hours.len; i++ ) {
+			sprintf(data,_("%02dH"),hours.list[ i ]);
+			fprintf(fp_ou, "<td class=\"header_c\">%s<br>%s</td>\n", data, label );
+		}
 		fprintf(fp_ou, "<td class=\"header_c\">%s<br>%s</td></tr>\n", _("TOTAL"), label );
 
 		memset(tttime,0,sizeof(tttime));
@@ -220,6 +232,13 @@ void report_day(const struct userinfostruct *uinfo)
 		}
 		fprintf(fp_ou, "<td class=\"header_r\">%s</td></tr>\n",fixtime(tt));
 		fputs("</table>\n",fp_ou);
+#ifdef ENABLE_DOUBLE_CHECK_DATA
+		if (tt!=uinfo->elap) {
+			debuga(_("Total elapsed time is %"PRIi64" instead of %"PRIi64" in the hourly report of user %s\n"),
+						 (int64_t)tt,(int64_t)uinfo->elap,uinfo->label);
+			exit(EXIT_FAILURE);
+		}
+#endif
 	}
 
 	if (write_html_trailer(fp_ou)<0)
