@@ -1,6 +1,6 @@
 /*
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
- *                                                            1998, 2011
+ *                                                            1998, 2012
  *
  * SARG donations:
  *      please look at http://sarg.sourceforge.net/donations.php
@@ -32,10 +32,8 @@ struct globalstatstruct globstat;
 
 static FILE *fp_tt=NULL;
 
-static void maketmp(const char *user, const char *dirname, int debug);
-static void maketmp_hour(const char *user, const char *dirname);
-static void gravatmp_hora(const char *dirname, const struct userinfostruct *uinfo, const char *data, const char *hora, long long int elap, long long int accbytes);
-static void gravatmp(const struct userinfostruct *uinfo, const char *oldurl, long long int nacc, long long int nbytes, const char *oldmsg, long long int nelap, long long int incache, long long int oucache);
+static FILE *maketmp(const char *user, const char *dirname, int debug);
+static void gravatmp(FILE *fp_ou, const char *oldurl, long long int nacc, long long int nbytes, const char *oldmsg, long long int nelap, long long int incache, long long int oucache);
 static void gravaporuser(const struct userinfostruct *uinfo, const char *dirname, const char *url, const char *ip, const char *data, const char *hora, long long int tam, long long int elap);
 static void gravager(FILE *fp_gen,const char *filename, const struct userinfostruct *uinfo, long long int nacc, const char *url, long long int nbytes, const char *ip, const char *hora, const char *dia, long long int nelap, long long int incache, long long int oucache);
 static void grava_SmartFilter(const char *dirname, const char *user, const char *ip, const char *data, const char *hora, const char *url, const char *smart);
@@ -44,6 +42,7 @@ void gerarel(void)
 {
 	FILE *fp_in;
 	FILE *fp_gen;
+	FILE *fp_tmp=NULL;
 
 	char *buf;
 	char accdia[11], acchora[9], accip[MAXLEN], *accurl;
@@ -63,7 +62,6 @@ void gerarel(void)
 	long long int nbytes=0;
 	long long int nelap=0;
 	long long int nacc=0;
-	long long int rtotal=0;
 	long long int incache=0;
 	long long int oucache=0;
 	long long int accbytes, accelap;
@@ -76,10 +74,11 @@ void gerarel(void)
 	int ourl_size=0;
 	int ourltt_size=0;
 	int same_url;
-	int new_user;
+	bool new_user;
 	struct getwordstruct gwarea;
 	longline line;
-	struct userinfostruct *uinfo,*puinfo;
+	struct userinfostruct *uinfo;
+	DayObject daystat;
 
 	ipantes[0]='\0';
 	smartfilter=0;
@@ -102,18 +101,23 @@ void gerarel(void)
 		exit(EXIT_FAILURE);
 	}
 
-	puinfo=NULL;
 	fp_tt=NULL;
 	sort_labels(&sort_field,&sort_order);
 
+	if (indexonly || datetimeby==0)
+		daystat=NULL;
+	else
+		daystat=day_prepare();
+	
 	uscan=userinfo_startscan();
 	if (uscan == NULL) {
 		debuga(_("Cannot enumerate the user list\n"));
 		exit(EXIT_FAILURE);
 	}
 	while ((uinfo = userinfo_advancescan(uscan)) != NULL ) {
-		if (snprintf(tmp3,sizeof(tmp3),"%s/%s.log",tmp,uinfo->filename)>=sizeof(tmp3)) {
-			debuga(_("(report) directory entry too long: %s/%s.log\n"),tmp,uinfo->filename);
+		sort_users_log(tmp,debug,uinfo);
+		if (snprintf(tmp3,sizeof(tmp3),"%s/%s.user_log",tmp,uinfo->filename)>=sizeof(tmp3)) {
+			debuga(_("(report) directory entry too long: %s/%s.user_log\n"),tmp,uinfo->filename);
 			exit(EXIT_FAILURE);
 		}
 		if((fp_in=MY_FOPEN(tmp3,"r"))==NULL){
@@ -121,6 +125,7 @@ void gerarel(void)
 			exit(EXIT_FAILURE);
 		}
 		user=uinfo->filename;
+		day_newuser(daystat);
 
 		strcpy(u2,uinfo->id);
 		if(Ip2Name && uinfo->id_is_ip) {
@@ -131,16 +136,21 @@ void gerarel(void)
 		user_find(uinfo->label,MAX_USER_LEN, u2);
 
 		if (!indexonly) {
-			maketmp(user,tmp,debug);
-			maketmp_hour(user,tmp);
+			fp_tmp=maketmp(user,tmp,debug);
 		}
 
 		ttopen=0;
+		oldurl=NULL;
 		oldurltt=NULL;
 		ourltt_size=0;
 		memset(oldaccdiatt,0,sizeof(oldaccdiatt));
 		memset(oldacchoratt,0,sizeof(oldacchoratt));
-		new_user=1;
+		new_user=true;
+		nacc=0;
+		nbytes=0;
+		nelap=0;
+		incache=0;
+		oucache=0;
 
 		if ((line=longline_create())==NULL) {
 			debuga(_("Not enough memory to read the downloaded files\n"));
@@ -168,7 +178,7 @@ void gerarel(void)
 
 			if(accsmart[0] != '\0') {
 				smartfilter++;
-				grava_SmartFilter(outdirname,uinfo->id,accip,accdia,acchora,accurl,accsmart);
+				grava_SmartFilter(tmp,uinfo->id,accip,accdia,acchora,accurl,accsmart);
 			}
 
 			if(Ip2Name) {
@@ -180,12 +190,11 @@ void gerarel(void)
 			}
 
 			if (!indexonly) {
-				gravatmp_hora(outdirname,uinfo,accdia,acchora,accelap,accbytes);
-
-				if(iprel) gravaporuser(uinfo,outdirname,accurl,accip,accdia,acchora,accbytes,accelap);
+				day_addpoint(daystat,accdia,acchora,accelap,accbytes);
+				if (iprel) gravaporuser(uinfo,outdirname,accurl,accip,accdia,acchora,accbytes,accelap);
 			}
 
-			if(!rtotal){
+			if(new_user){
 				url_len=strlen(accurl);
 				if (!oldurl || url_len>=ourl_size) {
 					ourl_size=url_len+1;
@@ -197,37 +206,21 @@ void gerarel(void)
 				}
 				strcpy(oldurl,accurl);
 				strcpy(oldacccode,acccode);
-				puinfo=uinfo;
 				strcpy(oldaccip,accip);
 				strcpy(oldaccdia,accdia);
 				strcpy(oldacchora,acchora);
-				new_user=0;
-				rtotal++;
+				new_user=false;
 			}
 			same_url=(strcmp(oldurl,accurl) == 0);
 
-			if(site[0] != '\0') {
-				if(new_user){
+			if(site[0] == '\0') {
+				if(!same_url){
 					if(strstr(oldacccode,"DENIED") != 0)
 						strcpy(oldmsg,"DENIED");
 					else
 						strcpy(oldmsg,"OK");
-					if (!indexonly) gravatmp(puinfo,oldurl,nacc,nbytes,oldmsg,nelap,incache,oucache);
-					gravager(fp_gen,wdirname,puinfo,nacc,oldurl,nbytes,oldaccip,oldacchora,oldaccdia,nelap,incache,oucache);
-					nacc=0;
-					nbytes=0;
-					nelap=0;
-					incache=0;
-					oucache=0;
-				}
-			} else {
-				if(!same_url || new_user){
-					if(strstr(oldacccode,"DENIED") != 0)
-						strcpy(oldmsg,"DENIED");
-					else
-						strcpy(oldmsg,"OK");
-					if (!indexonly) gravatmp(puinfo,oldurl,nacc,nbytes,oldmsg,nelap,incache,oucache);
-					gravager(fp_gen,wdirname,puinfo,nacc,oldurl,nbytes,oldaccip,oldacchora,oldaccdia,nelap,incache,oucache);
+					if (fp_tmp) gravatmp(fp_tmp,oldurl,nacc,nbytes,oldmsg,nelap,incache,oucache);
+					gravager(fp_gen,wdirname,uinfo,nacc,oldurl,nbytes,oldaccip,oldacchora,oldaccdia,nelap,incache,oucache);
 					nacc=0;
 					nbytes=0;
 					nelap=0;
@@ -297,14 +290,9 @@ void gerarel(void)
 			if (str) *str='\0';
 			if(strstr(crc2,"MISS") != 0)
 				oucache+=accbytes;
-			else incache+=accbytes;
+			else
+				incache+=accbytes;
 
-			if(new_user) {
-				new_user=0;
-				if (!indexonly) day_totalize(tmp,puinfo);
-			}
-
-			puinfo=uinfo;
 			strcpy(oldacccode,acccode);
 			strcpy(oldaccip,accip);
 			if (!same_url) {
@@ -321,30 +309,38 @@ void gerarel(void)
 			}
 			strcpy(oldaccdia,accdia);
 			strcpy(oldacchora,acchora);
-
 		}
 		fclose(fp_in);
 		longline_destroy(&line);
 		if (oldurltt) free(oldurltt);
-		if (unlink(tmp3)) {
-			debuga(_("Cannot delete %s - %s\n"),tmp3,strerror(errno));
+		if (oldurl) {
+			if(strstr(oldacccode,"DENIED") != 0)
+				strcpy(oldmsg,"DENIED");
+			else
+				strcpy(oldmsg,"OK");
+			if (fp_tmp) gravatmp(fp_tmp,oldurl,nacc,nbytes,oldmsg,nelap,incache,oucache);
+			gravager(fp_gen,wdirname,uinfo,nacc,oldurl,nbytes,oldaccip,oldacchora,oldaccdia,nelap,incache,oucache);
+			free(oldurl);
+			oldurl=NULL;
+		}
+		if(!new_user) {
+			day_totalize(daystat,tmp,uinfo);
+		}
+		if (fp_tmp) {
+			if (fclose(fp_tmp)==EOF)
+				debuga(_("Failed to close temporary file for user %s - %s\n"),uinfo->filename,strerror(errno));
+			fp_tmp=NULL;
+		}
+		if (!KeepTempLog && unlink(tmp3)) {
+			debuga(_("Cannot delete \"%s\": %s\n"),tmp3,strerror(errno));
 			exit(EXIT_FAILURE);
 		}
 	}
 	userinfo_stopscan(uscan);
+	day_cleanup(daystat);
 
-	if (oldurl) {
-		if(strstr(oldacccode,"DENIED") != 0)
-			strcpy(oldmsg,"DENIED");
-		else
-			strcpy(oldmsg,"OK");
-		if (!indexonly) gravatmp(puinfo,oldurl,nacc,nbytes,oldmsg,nelap,incache,oucache);
-		gravager(fp_gen,wdirname,puinfo,nacc,oldurl,nbytes,oldaccip,oldacchora,oldaccdia,nelap,incache,oucache);
-		free(oldurl);
-	}
 	totalger(fp_gen,wdirname);
 	fclose(fp_gen);
-	if (puinfo && !indexonly) day_totalize(tmp,puinfo);
 
 	if(email[0] == '\0') {
 		if (!indexonly) {
@@ -413,12 +409,13 @@ void gerarel(void)
 	return;
 }
 
-static void maketmp(const char *user, const char *dirname, int debug)
+static FILE *maketmp(const char *user, const char *dirname, int debug)
 {
 	FILE *fp_ou;
 	char wdirname[MAXLEN];
 
-	if((ReportType & REPORT_TYPE_USERS_SITES) == 0) return;
+	if((ReportType & REPORT_TYPE_USERS_SITES) == 0) return(NULL);
+	if((ReportType & REPORT_TYPE_TOPUSERS) == 0) return(NULL);
 
 	if(debug) debuga(_("Making file: %s/%s\n"),tmp,user);
 	if (snprintf(wdirname,sizeof(wdirname),"%s/%s.utmp",tmp,user)>=sizeof(wdirname)) {
@@ -431,61 +428,18 @@ static void maketmp(const char *user, const char *dirname, int debug)
 		exit(EXIT_FAILURE);
 	}
 
-	fclose(fp_ou);
-	return;
+	return(fp_ou);
 }
 
 
-static void maketmp_hour(const char *user, const char *dirname)
+static void gravatmp(FILE *fp_ou, const char *oldurl, long long int nacc, long long int nbytes, const char *oldmsg, long long int nelap, long long int incache, long long int oucache)
 {
-	FILE *fp_ou;
-	char wdirname[MAXLEN];
-
-	if((ReportType & REPORT_TYPE_USERS_SITES) == 0) return;
-	if(datetimeby==0) return;
-
-	if (snprintf(wdirname,sizeof(wdirname),"%s/%s.htmp",tmp,user)>=sizeof(wdirname)) {
-		debuga(_("Temporary file name too long: %s/%s.htmp\n"),tmp,user);
-		exit(EXIT_FAILURE);
-	}
-
-	if((fp_ou=fopen(wdirname,"w"))==NULL){
-		debuga(_("(report-1) Cannot open file %s - %s\n"),wdirname,strerror(errno));
-		exit(EXIT_FAILURE);
-	}
-
-	fclose(fp_ou);
-	return;
-}
-
-
-static void gravatmp(const struct userinfostruct *uinfo, const char *oldurl, long long int nacc, long long int nbytes, const char *oldmsg, long long int nelap, long long int incache, long long int oucache)
-{
-	FILE *fp_ou;
-	char wdirname[MAXLEN];
-
-	if((ReportType & REPORT_TYPE_USERS_SITES) == 0) return;
-
-	if (snprintf(wdirname,sizeof(wdirname),"%s/%s.utmp",tmp,uinfo->filename)>=sizeof(wdirname)) {
-		debuga(_("Temporary file name too long: %s/%s.utmp\n"),tmp,uinfo->filename);
-		exit(EXIT_FAILURE);
-	}
-
-	if((fp_ou=MY_FOPEN(wdirname,"a"))==NULL){
-		debuga(_("(report) Cannot open file %s\n"),wdirname);
-		exit(EXIT_FAILURE);
-	}
-
 	/*
 	This complicated printf is due to Microsoft's inability to comply with any standard. Msvcrt is unable
 	to print a long long int unless it is exactly 64-bits long.
 	*/
 	fprintf(fp_ou,"%"PRIu64"\t%"PRIu64"\t%s\t%s\t%"PRIu64"\t%"PRIu64"\t%"PRIu64"\n",(uint64_t)nacc,(uint64_t)nbytes,oldurl,oldmsg,(uint64_t)nelap,(uint64_t)incache,(uint64_t)oucache);
 
-	if (fclose(fp_ou)==EOF) {
-		debuga(_("Failed to close file %s - %s\n"),wdirname,strerror(errno));
-		exit(EXIT_FAILURE);
-	}
 	ttopen=0;
 
 	if(fp_tt) {
@@ -498,39 +452,6 @@ static void gravatmp(const struct userinfostruct *uinfo, const char *oldurl, lon
 	return;
 }
 
-static void gravatmp_hora(const char *dirname, const struct userinfostruct *uinfo, const char *data, const char *hora, long long int elap, long long int bytes)
-{
-	FILE *fp_ou;
-	char wdirname[MAXLEN];
-	int hour;
-
-	if(((ReportType & REPORT_TYPE_USERS_SITES) == 0) || datetimeby==0) return;
-
-	if (snprintf(wdirname,sizeof(wdirname),"%s/%s.htmp",tmp,uinfo->filename)>=sizeof(wdirname)) {
-		debuga(_("Path too long %s/%s.htmp\n"),tmp,uinfo->filename);
-		exit(EXIT_FAILURE);
-	}
-
-	if((fp_ou=MY_FOPEN(wdirname,"a"))==NULL){
-		debuga(_("(report-2) Cannot open file %s - %s\n"),wdirname,strerror(errno));
-		exit(EXIT_FAILURE);
-	}
-
-	hour=atoi(hora);
-	fprintf(fp_ou,"%s\t%d",data,hour);
-	if((datetimeby & DATETIME_BYTE)!=0) fprintf(fp_ou,"\t%"PRIu64,(uint64_t)bytes);
-	if((datetimeby & DATETIME_ELAP)!=0) fprintf(fp_ou,"\t%"PRIu64,(uint64_t)elap);
-	fputs("\n",fp_ou);
-
-	if (fclose(fp_ou)==EOF) {
-		debuga(_("Failed to close file %s - %s\n"),wdirname,strerror(errno));
-		exit(EXIT_FAILURE);
-	}
-
-	return;
-}
-
-
 static void gravaporuser(const struct userinfostruct *uinfo, const char *dirname, const char *url, const char *ip, const char *data, const char *hora, long long int tam, long long int elap)
 {
 	FILE *fp_ou;
@@ -822,7 +743,10 @@ static void grava_SmartFilter(const char *dirname, const char *user, const char
 	FILE *fp_ou;
 	char wdirname[MAXLEN];
 
-	sprintf(wdirname,"%s/smartfilter.unsort",dirname);
+	if (snprintf(wdirname,sizeof(wdirname),"%s/smartfilter.int_unsort",dirname)>=sizeof(wdirname)) {
+		debuga(_("File name too long for %s/smartfilter.int_unsort\n"),dirname);
+		exit(EXIT_FAILURE);
+	}
 
 	if((fp_ou=MY_FOPEN(wdirname,"a"))==NULL){
 		debuga(_("(report) Cannot open file %s\n"),wdirname);
