@@ -1,6 +1,6 @@
 /*
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
- *                                                            1998, 2011
+ *                                                            1998, 2012
  *
  * SARG donations:
  *      please look at http://sarg.sourceforge.net/donations.php
@@ -97,6 +97,7 @@ void siteuser(void)
 		fprintf(fp_ou,"<th class=\"header_l\">%s</th>",_("BYTES"));
 	fputs("<th class=\"header_l",fp_ou);
 	if (SortTableJs[0]) fputs(" sorttable_alpha",fp_ou);
+	/* TRANSLATORS: This is a column header showing the users who visited each site. */
 	fprintf(fp_ou,"\">%s</th></tr></thead>\n",_("USERS"));
 
 	ourl=NULL;
@@ -180,8 +181,8 @@ void siteuser(void)
 	}
 	if (ourl) free(ourl);
 
-	if (unlink(general2)) {
-		debuga(_("Cannot delete %s - %s\n"),general2,strerror(errno));
+	if (!KeepTempLog && unlink(general2)) {
+		debuga(_("Cannot delete \"%s\": %s\n"),general2,strerror(errno));
 		exit(EXIT_FAILURE);
 	}
 
