@@ -1,6 +1,6 @@
 /*
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
- *                                                            1998, 2011
+ *                                                            1998, 2012
  *
  * SARG donations:
  *      please look at http://sarg.sourceforge.net/donations.php
@@ -53,16 +53,16 @@ void smartfilter_report(void)
 
 	ouser[0]='\0';
 
-	if (snprintf(smart_in,sizeof(smart_in),"%s/smartfilter.unsort",outdirname)>=sizeof(smart_in)) {
-		debuga(_("File name too long: %s/smartfilter.unsort\n"),outdirname);
+	if (snprintf(smart_in,sizeof(smart_in),"%s/smartfilter.int_unsort",tmp)>=sizeof(smart_in)) {
+		debuga(_("File name too long: %s/smartfilter.int_unsort\n"),tmp);
 		exit(EXIT_FAILURE);
 	}
 	if (snprintf(sites,sizeof(sites),"%s/sarg-sites",outdirname)>=sizeof(sites)) {
 		debuga(_("File name too long: %s/sarg-sites\n"),outdirname);
 		exit(EXIT_FAILURE);
 	}
-	if (snprintf(smart_ou,sizeof(smart_ou),"%s/smartfilter.log",outdirname)>=sizeof(smart_ou)) {
-		debuga(_("File name too long: %s/smartfilter.log\n"),outdirname);
+	if (snprintf(smart_ou,sizeof(smart_ou),"%s/smartfilter.int_log",tmp)>=sizeof(smart_ou)) {
+		debuga(_("File name too long: %s/smartfilter.int_log\n"),tmp);
 		exit(EXIT_FAILURE);
 	}
 	if (snprintf(report,sizeof(report),"%s/smartfilter.html",outdirname)>=sizeof(report)) {
@@ -85,8 +85,8 @@ void smartfilter_report(void)
 		debuga(_("sort command: %s\n"),csort);
 		exit(EXIT_FAILURE);
 	}
-	if (unlink(smart_in)) {
-		debuga(_("Cannot delete %s - %s\n"),smart_in,strerror(errno));
+	if (!KeepTempLog && unlink(smart_in)) {
+		debuga(_("Cannot delete \"%s\": %s\n"),smart_in,strerror(errno));
 		exit(EXIT_FAILURE);
 	}
 
@@ -207,5 +207,10 @@ void smartfilter_report(void)
 		fclose(fp_user);
 	}
 
+	if (!KeepTempLog && unlink(smart_ou)) {
+		debuga(_("Cannot delete \"%s\": %s\n"),smart_ou,strerror(errno));
+		exit(EXIT_FAILURE);
+	}
+
 	return;
 }
