@@ -1,6 +1,6 @@
 /*
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
- *                                                            1998, 2011
+ *                                                            1998, 2012
  *
  * SARG donations:
  *      please look at http://sarg.sourceforge.net/donations.php
@@ -78,7 +78,9 @@ void splitlog(const char *arq, const char *df, int dfrom, int duntil, int conver
 	if(arq[0] == '\0')
 		arq="/var/log/squid/access.log";
 
-	if((fp_in=MY_FOPEN(arq,"r"))==NULL) {
+	if (arq[0]=='-' && arq[1]=='\0') {
+		fp_in=stdin;
+	} else if((fp_in=MY_FOPEN(arq,"r"))==NULL) {
 		debuga(_("(splitlog) Cannot open log file %s - %s\n"),arq,strerror(errno));
 		exit(EXIT_FAILURE);
 	}
@@ -139,7 +141,7 @@ void splitlog(const char *arq, const char *df, int dfrom, int duntil, int conver
 	}
 
 	longline_destroy(&line);
-	if (fclose(fp_in)==EOF) {
+	if (fp_in!=stdin && fclose(fp_in)==EOF) {
 		debuga(_("Failed to close file %s - %s\n"),arq,strerror(errno));
 	}
 	if (autosplit && fp_ou) {
