@@ -1,6 +1,6 @@
 /*
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
- *                                                            1998, 2011
+ *                                                            1998, 2012
  *
  * SARG donations:
  *      please look at http://sarg.sourceforge.net/donations.php
@@ -44,6 +44,7 @@ void topsites(void)
 	char general3[MAXLEN];
 	char sites[MAXLEN];
 	char report[MAXLEN];
+	char ouser[MAX_USER_LEN]="";
 	const char *sortf;
 	const char *sortt;
 	long long int nacc;
@@ -58,6 +59,7 @@ void topsites(void)
 	long long int ttnbytes=0;
 	long long int ttntime=0;
 #endif
+	int nusers=0;
 	int regs=0;
 	int cstatus;
 	int url_len;
@@ -76,12 +78,9 @@ void topsites(void)
 	sprintf(general2,"%s/sarg-general2",outdirname);
 	sprintf(general3,"%s/sarg-general3",outdirname);
 
-	if ((ReportType & REPORT_TYPE_TOPUSERS) == 0)
-		sprintf(report,"%s/index.html",outdirname);
-	else
-		sprintf(report,"%s/topsites.html",outdirname);
+	sprintf(report,"%s/topsites.html",outdirname);
 
-	if (snprintf(csort,sizeof(csort),"sort -t \"\t\" -k 4,4 -o \"%s\" \"%s\"",general2,general)>=sizeof(csort)) {
+	if (snprintf(csort,sizeof(csort),"sort -t \"\t\" -k 4,4 -k 1,1 -o \"%s\" \"%s\"",general2,general)>=sizeof(csort)) {
 		debuga(_("Sort command too long when sorting file \"%s\" to \"%s\"\n"),general,general2);
 		exit(EXIT_FAILURE);
 	}
@@ -131,7 +130,7 @@ void topsites(void)
 			This complicated printf is due to Microsoft's inability to comply with any standard. Msvcrt is unable
 			to print a long long int unless it is exactly 64-bits long.
 			*/
-			fprintf(fp_ou,"%"PRIu64"\t%"PRIu64"\t%"PRIu64"\t%s\n",(uint64_t)tnacc,(uint64_t)tnbytes,(uint64_t)tntime,ourl);
+			fprintf(fp_ou,"%"PRIu64"\t%"PRIu64"\t%"PRIu64"\t%d\t%s\n",(uint64_t)tnacc,(uint64_t)tnbytes,(uint64_t)tntime,nusers,ourl);
 			url_len=strlen(item.url);
 			if (url_len>=ourl_size) {
 				ourl_size=url_len+1;
@@ -142,9 +141,14 @@ void topsites(void)
 				}
 			}
 			strcpy(ourl,item.url);
+			strcpy(ouser,item.user);
 			tnacc=0;
 			tnbytes=0;
 			tntime=0;
+			nusers=1;
+		} else if (strcmp(item.user,ouser)!=0) {
+			strcpy(ouser,item.user);
+			nusers++;
 		}
 
 		tnacc+=item.nacc;
@@ -164,7 +168,7 @@ void topsites(void)
 		This complicated printf is due to Microsoft's inability to comply with any standard. Msvcrt is unable
 		to print a long long int unless it is exactly 64-bits long.
 		*/
-		fprintf(fp_ou,"%"PRIu64"\t%"PRIu64"\t%"PRIu64"\t%s\n",(uint64_t)tnacc,(uint64_t)tnbytes,(uint64_t)tntime,ourl);
+		fprintf(fp_ou,"%"PRIu64"\t%"PRIu64"\t%"PRIu64"\t%d\t%s\n",(uint64_t)tnacc,(uint64_t)tnbytes,(uint64_t)tntime,nusers,ourl);
 		free(ourl);
 	}
 
@@ -177,8 +181,8 @@ void topsites(void)
 	}
 #endif
 
-	if (unlink(general2)) {
-		debuga(_("Cannot delete %s - %s\n"),general2,strerror(errno));
+	if (!KeepTempLog && unlink(general2)) {
+		debuga(_("Cannot delete \"%s\": %s\n"),general2,strerror(errno));
 		exit(EXIT_FAILURE);
 	}
 
@@ -188,6 +192,8 @@ void topsites(void)
 		sortf="-k 2,2 -k 1,1";
 	} else if((TopsitesSort & TOPSITE_SORT_TIME) != 0) {
 		sortf="-k 3,3";
+	} else if((TopsitesSort & TOPSITE_SORT_USER) != 0) {
+		sortf="-k 4,4 -k 1,1 -k 2,2";
 	} else {
 		sortf="-k 2,2 -k 1,1"; //default is BYTES
 	}
@@ -213,8 +219,8 @@ void topsites(void)
 		exit(EXIT_FAILURE);
 	}
 
-	if (unlink(general3)) {
-		debuga(_("Cannot delete %s - %s\n"),general3,strerror(errno));
+	if (!KeepTempLog && unlink(general3)) {
+		debuga(_("Cannot delete \"%s\": %s\n"),general3,strerror(errno));
 		exit(EXIT_FAILURE);
 	}
 
@@ -235,9 +241,21 @@ void topsites(void)
 	fputs("<div class=\"report\"><table cellpadding=\"1\" cellspacing=\"2\"",fp_ou);
 	if (SortTableJs[0]) fputs(" class=\"sortable\"",fp_ou);
 	fputs(">\n",fp_ou);
-	fprintf(fp_ou,"<thead><tr><th class=\"header_l\">%s</th><th class=\"header_l",_("NUM"));
+	fprintf(fp_ou,"<thead><tr><th class=\"header_l\">%s</th><th class=\"header_l",
+	/* TRANSLATORS: This is a column header showing the position of the entry in the sorted list. */
+	_("NUM"));
 	if (SortTableJs[0]) fputs(" sorttable_alpha",fp_ou);
-	fprintf(fp_ou,"\">%s</th><th class=\"header_l\">%s</th><th class=\"header_l\">%s</th><th class=\"header_l\">%s</th></tr></thead>\n",_("ACCESSED SITE"),_("CONNECT"),_("BYTES"),_("TIME"));
+	fprintf(fp_ou,"\">%s</th><th class=\"header_l\">%s</th><th class=\"header_l\">%s</th><th class=\"header_l\">%s</th><th class=\"header_l\">%s</th></tr></thead>\n",
+	/* TRANSLATORS: This is a column header showing the URL of the visited sites. */
+	_("ACCESSED SITE"),
+	/* TRANSLATORS: This is a column header showing the number of connections to a visited site. */
+	_("CONNECT"),
+	/* TRANSLATORS: This is a column header showing the number of transfered bytes. */
+	_("BYTES"),
+	/* TRANSLATORS: This is a column header showing the time spent by the proxy processing the requests. */
+	_("TIME"),
+	/* TRANSLATORS: This is a column header showing the number of users who visited a sites. */
+	_("USERS"));
 
 	regs=0;
 	ntopsites = 0;
@@ -258,6 +276,10 @@ void topsites(void)
 			debuga(_("Maybe you have a broken record or garbage in your %s file\n"),sites);
 			exit(EXIT_FAILURE);
 		}
+		if (getword_atoi(&nusers,&gwarea,'\t')<0) {
+			debuga(_("The number of users is invalid in file %s\n"),sites);
+			exit(EXIT_FAILURE);
+		}
 		if (getword_ptr(buf,&url,&gwarea,'\t')<0) {
 			debuga(_("The url is invalid in file %s\n"),sites);
 			exit(EXIT_FAILURE);
@@ -284,7 +306,10 @@ void topsites(void)
 		fprintf(fp_ou,">%s</td>",fixnum(twork2,1));
 		fputs("<td class=\"data\"",fp_ou);
 		if (SortTableJs[0]) fprintf(fp_ou," sorttable_customkey=\"%"PRId64"\"",(uint64_t)twork3);
-		fprintf(fp_ou,">%s</td></tr>\n",fixtime(twork3));
+		fprintf(fp_ou,">%s</td>",fixtime(twork3));
+		fputs("<td class=\"data\"",fp_ou);
+		if (SortTableJs[0]) fprintf(fp_ou," sorttable_customkey=\"%d\"",nusers);
+		fprintf(fp_ou,">%s</td></tr>\n",fixnum(nusers,1));
 	}
 	fclose(fp_in);
 	longline_destroy(&line);
