@@ -1,6 +1,6 @@
 /*
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
- *                                                            1998, 2011
+ *                                                            1998, 2012
  *
  * SARG donations:
  *      please look at http://sarg.sourceforge.net/donations.php
@@ -29,9 +29,31 @@
 
 extern struct globalstatstruct globstat;
 
+/*!
+Save the total number of users. The number is written in sarg-users and set
+in a global variable for further reference.
+
+\param totuser The total number of users.
+*/
+static void set_total_users(int totuser)
+{
+	char tusr[1024];
+	FILE *fp_ou;
+	
+	snprintf(tusr,sizeof(tusr),"%s/sarg-users",outdirname);
+	if((fp_ou=fopen(tusr,"w"))==NULL) {
+		debuga(_("(topuser) Cannot open file %s\n"),tusr);
+		exit(EXIT_FAILURE);
+	}
+	fprintf(fp_ou,"%d\n",totuser);
+	if (fclose(fp_ou)==EOF)
+		debuga(_("Failed to close file %s - %s\n"),tusr,strerror(errno));
+	globstat.totuser=totuser;
+}
+
 void topuser(void)
 {
-	FILE *fp_in = NULL, *fp_ou = NULL, *fp_top1 = NULL, *fp_top2 = NULL, *fp_top3 = NULL;
+	FILE *fp_in = NULL, *fp_top1 = NULL, *fp_top2 = NULL, *fp_top3 = NULL;
 	long long int ttnbytes=0, ttnacc=0, tnacc=0;
 	long long int tnbytes=0, ttnelap=0, tnelap=0;
 	long long int tnincache=0, tnoucache=0, ttnincache=0, ttnoucache=0;
@@ -44,7 +66,7 @@ void topuser(void)
 	int posicao=0;
 	char olduser[MAX_USER_LEN], csort[MAXLEN];
 	char wger[MAXLEN], top1[MAXLEN], top2[MAXLEN], top3[MAXLEN];
-	char user[MAX_USER_LEN], tusr[MAXLEN];
+	char user[MAX_USER_LEN];
 	const char *sfield="-n -k 2,2";
 	const char *order;
 	const char *sort_field;
@@ -139,6 +161,8 @@ void topuser(void)
 	}
 #endif
 
+	set_total_users(totuser);
+
 	if((TopuserSort & TOPUSER_SORT_USER) != 0) {
 		sfield="-k 1,1";
 		sort_field=_("user");
@@ -177,8 +201,8 @@ void topuser(void)
 		exit(EXIT_FAILURE);
 	}
 
-	if (unlink(top2)) {
-		debuga(_("Cannot delete %s - %s\n"),top2,strerror(errno));
+	if (!KeepTempLog && unlink(top2)) {
+		debuga(_("Cannot delete \"%s\": %s\n"),top2,strerror(errno));
 		exit(EXIT_FAILURE);
 	}
 
@@ -193,10 +217,17 @@ void topuser(void)
 	fputs("<tr><td class=\"header_c\">",fp_top3);
 	fprintf(fp_top3,_("Period: %s"),period.html);
 	fputs("</td></tr>\n",fp_top3);
-	fputs("<tr><td class=\"header_c\">",fp_top3);
-	fprintf(fp_top3,_("Sort: %s, %s"),sort_field,sort_order);
-	fputs("</td></tr>\n",fp_top3);
-	fprintf(fp_top3,"<tr><th class=\"header_c\">%s</th></tr>\n",_("Top users"));
+	if ((ReportType & REPORT_TYPE_TOPUSERS) != 0) {
+		fputs("<tr><td class=\"header_c\">",fp_top3);
+		fprintf(fp_top3,_("Sort: %s, %s"),sort_field,sort_order);
+		fputs("</td></tr>\n",fp_top3);
+		fprintf(fp_top3,"<tr><th class=\"header_c\">%s</th></tr>\n",_("Top users"));
+	} else {
+		/* TRANSLATORS: This is the title of the main report page when no
+		 * top users list are requested.
+		 */
+		fprintf(fp_top3,"<tr><th class=\"header_c\">%s</th></tr>\n",_("Table of content"));
+	}
 	close_html_header(fp_top3);
 
 	if (!indexonly) {
@@ -254,6 +285,8 @@ void topuser(void)
 
 	fputs("</tr></thead>\n",fp_top3);
 
+	greport_prepare();
+
 	ntopuser = 0;
 
 	if ((line=longline_create())==NULL) {
@@ -301,6 +334,9 @@ void topuser(void)
 		}
 		uinfo->topuser=1;
 
+		report_day(uinfo);
+		greport_day(uinfo);
+
 		fputs("<tr>",fp_top3);
 
 		posicao++;
@@ -320,7 +356,7 @@ void topuser(void)
 			} else {
 				sprintf(val1,"%s/d%s.html",outdirname,uinfo->filename);
 				if (unlink(val1)) {
-					debuga(_("Cannot delete %s - %s\n"),val1,strerror(errno));
+					debuga(_("Cannot delete \"%s\": %s\n"),val1,strerror(errno));
 					exit(EXIT_FAILURE);
 				}
 			}
@@ -350,7 +386,7 @@ void topuser(void)
 			ouperc=(tnbytes) ? oucac * 100. / tnbytes : 0.;
 			fprintf(fp_top3,"<td class=\"data\">%3.2lf%%</td><td class=\"data\">%3.2lf%%</td>",inperc,ouperc);
 #ifdef ENABLE_DOUBLE_CHECK_DATA
-			if (fabs(inperc+ouperc-100.)>=0.01) {
+			if ((inperc!=0. || ouperc!=0.) && fabs(inperc+ouperc-100.)>=0.01) {
 				debuga(_("The total of the in-cache and cache-miss is not 100%% at position %d (user %s)\n"),posicao,uinfo->label);
 			}
 #endif
@@ -375,8 +411,8 @@ void topuser(void)
 		topcount++;
 	}
 	fclose(fp_top1);
-	if (unlink(top1)) {
-		debuga(_("Cannot delete %s - %s\n"),top1,strerror(errno));
+	if (!KeepTempLog && unlink(top1)) {
+		debuga(_("Cannot delete \"%s\": %s\n"),top1,strerror(errno));
 		exit(EXIT_FAILURE);
 	}
 	longline_destroy(&line);
@@ -413,6 +449,7 @@ void topuser(void)
 
 		fputs("</tr>\n",fp_top3);
 	}
+	greport_cleanup();
 
 	if(ntopuser && (TopUserFields & TOPUSERFIELDS_AVERAGE) != 0) {
 		fputs("<tr>",fp_top3);
@@ -445,15 +482,5 @@ void topuser(void)
 	if (fclose(fp_top3)==EOF)
 		debuga(_("Failed to close the top user list %s - %s\n"),top3,strerror(errno));
 
-	snprintf(tusr,sizeof(tusr),"%s/sarg-users",outdirname);
-	if((fp_ou=fopen(tusr,"w"))==NULL) {
-		debuga(_("(topuser) Cannot open file %s\n"),tusr);
-		exit(EXIT_FAILURE);
-	}
-	fprintf(fp_ou,"%d\n",totuser);
-	if (fclose(fp_ou)==EOF)
-		debuga(_("Failed to close file %s - %s\n"),tusr,strerror(errno));
-	globstat.totuser=totuser;
-
 	return;
 }
