@@ -1,6 +1,6 @@
 /*
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
- *                                                            1998, 2011
+ *                                                            1998, 2012
  *
  * SARG donations:
  *      please look at http://sarg.sourceforge.net/donations.php
@@ -26,6 +26,10 @@
 
 #include "include/conf.h"
 #include "include/defs.h"
+#ifdef HAVE_PCRE_H
+#include <pcre.h>
+#define USE_PCRE
+#endif
 
 /*!
 A host name and the name to report.
@@ -78,6 +82,23 @@ struct hostalias_ipv6
 	const char *Alias;
 };
 
+#ifdef USE_PCRE
+/*!
+A regular expression.
+*/
+struct hostalias_regex
+{
+	//! The next regular expression to test.
+	struct hostalias_regex *Next;
+	//! The regular expression to match against the host name.
+	pcre *Re;
+	//! The replacement name.
+	const char *Alias;
+	//! \c True if this regular expression contains at least one subpattern
+	bool SubPartern;
+};
+#endif
+
 //! The first host name.
 static struct hostalias_name *FirstAliasName=NULL;
 //! The first IPv4 address.
@@ -85,6 +106,10 @@ static struct hostalias_ipv4 *FirstAliasIpv4=NULL;
 //! The first IPv§ address.
 static struct hostalias_ipv6 *FirstAliasIpv6=NULL;
 
+#ifdef USE_PCRE
+static struct hostalias_regex *FirstAliasRe=NULL;
+#endif
+
 /*!
   Store a name to alias.
 
@@ -410,6 +435,137 @@ static int Alias_StoreIpv6(unsigned short *ipv6,int nbits,const char *next)
 	return(1);
 }
 
+#ifdef USE_PCRE
+/*!
+Store a regular expression to match the alias.
+
+\retval 1 Alias added.
+\retval 0 Ignore the line.
+\retval -1 Error.
+*/
+static int Alias_StoreRegexp(char *buf)
+{
+	char Delimiter;
+	char *End;
+	struct hostalias_regex *alias;
+	struct hostalias_regex *new_alias;
+	struct hostalias_regex **prev_alias;
+	const char *PcreError;
+	int ErrorOffset;
+	char *Replace;
+	int len;
+	char *tmp;
+	int i;
+	
+	// find the pattern
+	Delimiter=*buf++;
+	for (End=buf ; *End && *End!=Delimiter ; End++) {
+		if (*End=='\\') {
+			if (End[1]=='\0') {
+				debuga(_("Invalid NUL character found in regular expression\n"));
+				return(-1);
+			}
+			End++; //ignore the escaped character
+		}
+	}
+	if (*End!=Delimiter) {
+		debuga(_("Unterminated regular expression\n"));
+		return(-1);
+	}
+	*End++='\0';
+	
+	// find the alias
+	for (Replace=End ; *Replace==' ' || *Replace=='\t' ; Replace++);
+	for (End=Replace ; *End && (unsigned char)*End>' ' ; End++);
+	*End='\0';
+		
+	// store it
+	new_alias=malloc(sizeof(*new_alias));
+	if (!new_alias) {
+		debuga(_("Not enough memory to store the host name aliasing directives\n"));
+		return(-1);
+	}
+	new_alias->Next=NULL;
+	new_alias->Re=pcre_compile(buf,0,&PcreError,&ErrorOffset,NULL);
+	if (new_alias->Re==NULL) {
+		debuga(_("Failed to compile the regular expression \"%s\": %s\n"),buf,PcreError);
+		free(new_alias);
+		return(-1);
+	}
+	len=strlen(Replace);
+	tmp=malloc(len+2);
+	if (!tmp) {
+		debuga(_("Not enough memory to store the host name aliasing directives\n"));
+		pcre_free(new_alias->Re);
+		return(-1);
+	}
+	tmp[0]=ALIAS_PREFIX;
+	memcpy(tmp+1,Replace,len);
+	tmp[len+1]='\0';
+	new_alias->Alias=tmp;
+	
+	new_alias->SubPartern=false;
+	for (i=1 ; tmp[i] ; i++)
+		// both the sed \1 and the perl $1 replacement operators are accepted
+		if ((tmp[i]=='\\' || tmp[i]=='$') && isdigit(tmp[i+1])) {
+			new_alias->SubPartern=true;
+			break;
+		}
+	
+	// chain it
+	prev_alias=&FirstAliasRe;
+	for (alias=FirstAliasRe ; alias ; alias=alias->Next)
+		prev_alias=&alias->Next;
+	*prev_alias=new_alias;
+		
+	return(1);
+}
+#endif
+
+/*!
+Store an alias in the corresponding list.
+
+\param buf The string to parse and store.
+
+\retval 0 No error.
+\retval -1 Error in file.
+*/
+static int Alias_Store(char *buf)
+{
+	int type;
+	const char *name;
+	unsigned char ipv4[4];
+	unsigned short int ipv6[8];
+	int nbits;
+	const char *next;
+	int Error;
+	
+	if (strncasecmp(buf,"re:",3)==0) {
+#ifdef USE_PCRE
+		if (Alias_StoreRegexp(buf+3)<0)
+			return(-1);
+		return(0);
+#else
+		debuga(_("PCRE not compiled in therefore the regular expressions are not available in the host alias file\n"));
+		return(-1);
+#endif
+	}
+	type=extract_address_mask(buf,&name,ipv4,ipv6,&nbits,&next);
+	if (type<0) {
+		return(-1);
+	}
+
+	if (type==1) {
+		Error=Alias_StoreName(name,next);
+	} else if (type==2) {
+		Error=Alias_StoreIpv4(ipv4,nbits,next);
+	} else if (type==3) {
+		Error=Alias_StoreIpv6(ipv6,nbits,next);
+	}
+	if (Error<0) return(-1);
+	return(0);
+}
+
 /*!
 Read the file containing the host names to alias in the report.
 
@@ -420,12 +576,6 @@ void read_hostalias(const char *Filename)
 	FILE *fi;
 	longline line;
 	char *buf;
-	int type;
-	const char *name;
-	unsigned char ipv4[4];
-	unsigned short int ipv6[8];
-	int nbits;
-	const char *next;
 
 	if (debug) debuga(_("Reading host alias file \"%s\"\n"),Filename);
 	fi=fopen(Filename,"rt");
@@ -440,19 +590,10 @@ void read_hostalias(const char *Filename)
 	}
 
 	while ((buf=longline_read(fi,line)) != NULL) {
-		type=extract_address_mask(buf,&name,ipv4,ipv6,&nbits,&next);
-		if (type<0) {
+		if (Alias_Store(buf)<0) {
 			debuga(_("While reading \"%s\"\n"),Filename);
 			exit(EXIT_FAILURE);
 		}
-
-		if (type==1) {
-			Alias_StoreName(name,next);
-		} else if (type==2) {
-			Alias_StoreIpv4(ipv4,nbits,next);
-		} else if (type==3) {
-			Alias_StoreIpv6(ipv6,nbits,next);
-		}
 	}
 	
 	longline_destroy(&line);
@@ -487,33 +628,55 @@ Free the memory allocated by read_hostalias().
 */
 void free_hostalias(void)
 {
-	struct hostalias_name *alias1;
-	struct hostalias_name *next1;
-	struct hostalias_ipv4 *alias4;
-	struct hostalias_ipv4 *next4;
-	struct hostalias_ipv6 *alias6;
-	struct hostalias_ipv6 *next6;
-	
-	for (alias1=FirstAliasName ; alias1 ; alias1=next1) {
-		next1=alias1->Next;
-		if (alias1->HostName_Prefix) free((void *)alias1->HostName_Prefix);
-		if (alias1->HostName_Suffix) free((void *)alias1->HostName_Suffix);
-		free((void *)alias1->Alias);
-		free(alias1);
+	{
+		struct hostalias_name *alias1;
+		struct hostalias_name *next1;
+		
+		for (alias1=FirstAliasName ; alias1 ; alias1=next1) {
+			next1=alias1->Next;
+			if (alias1->HostName_Prefix) free((void *)alias1->HostName_Prefix);
+			if (alias1->HostName_Suffix) free((void *)alias1->HostName_Suffix);
+			free((void *)alias1->Alias);
+			free(alias1);
+		}
+		FirstAliasName=NULL;
 	}
-	FirstAliasName=NULL;
-	for (alias4=FirstAliasIpv4 ; alias4 ; alias4=next4) {
-		next4=alias4->Next;
-		free((void *)alias4->Alias);
-		free(alias4);
+	{
+		struct hostalias_ipv4 *alias4;
+		struct hostalias_ipv4 *next4;
+		
+		for (alias4=FirstAliasIpv4 ; alias4 ; alias4=next4) {
+			next4=alias4->Next;
+			free((void *)alias4->Alias);
+			free(alias4);
+		}
+		FirstAliasIpv4=NULL;
 	}
-	FirstAliasIpv4=NULL;
-	for (alias6=FirstAliasIpv6 ; alias6 ; alias6=next6) {
-		next6=alias6->Next;
-		free((void *)alias6->Alias);
-		free(alias6);
+	{
+		struct hostalias_ipv6 *alias6;
+		struct hostalias_ipv6 *next6;
+		
+		for (alias6=FirstAliasIpv6 ; alias6 ; alias6=next6) {
+			next6=alias6->Next;
+			free((void *)alias6->Alias);
+			free(alias6);
+		}
+		FirstAliasIpv6=NULL;
 	}
-	FirstAliasIpv6=NULL;
+#ifdef USE_PCRE
+	{
+		struct hostalias_regex *alias;
+		struct hostalias_regex *next;
+		
+		for (alias=FirstAliasRe ; alias ; alias=next) {
+			next=alias->Next;
+			pcre_free(alias->Re);
+			free((void *)alias->Alias);
+			free(alias);
+		}
+		FirstAliasRe=NULL;
+	}
+#endif
 }
 
 /*!
@@ -523,7 +686,7 @@ Replace the host name by its alias if it is in our list.
 
 \return The pointer to the host name or its alias.
 */
-const char *alias_url_name(const char *url,const char *next)
+static const char *alias_url_name(const char *url,const char *next)
 {
 	struct hostalias_name *alias;
 	int len;
@@ -557,7 +720,7 @@ Replace the IPv4 address by its alias if it is in our list.
 
 \return The pointer to the host name or its alias.
 */
-const char *alias_url_ipv4(const char *url,unsigned char *ipv4)
+static const char *alias_url_ipv4(const char *url,unsigned char *ipv4)
 {
 	struct hostalias_ipv4 *alias;
 	int len;
@@ -579,7 +742,7 @@ Replace the IPv6 address by its alias if it is in our list.
 
 \return The pointer to the host name or its alias.
 */
-const char *alias_url_ipv6(const char *url,unsigned short int *ipv6)
+static const char *alias_url_ipv6(const char *url,unsigned short int *ipv6)
 {
 	struct hostalias_ipv6 *alias;
 	int len;
@@ -597,6 +760,94 @@ const char *alias_url_ipv6(const char *url,unsigned short int *ipv6)
 	return(url);
 }
 
+#ifdef USE_PCRE
+/*!
+Replace the host name by its alias if it is in our list.
+
+\param url_ptr A pointer to the host name to match. It is replaced
+by a pointer to the alias if a match is found.
+
+\return \c True if a match is found or \c false if it failed.
+
+\warning The function is not thread safe as it may return a static
+internal buffer.
+*/
+static bool alias_url_regex(const char **url_ptr)
+{
+	struct hostalias_regex *alias;
+	int nmatches;
+	const char *url;
+	int url_len;
+	int ovector[30];//size must be a multiple of 3
+	static char Replacement[1024];
+	const char *str;
+	int i;
+	int sub;
+	int repl_idx;
+
+	url=*url_ptr;
+	url_len=strlen(url);
+	for (alias=FirstAliasRe ; alias ; alias=alias->Next) {
+		nmatches=pcre_exec(alias->Re,NULL,url,url_len,0,0,ovector,sizeof(ovector)/sizeof(ovector[0]));
+		if (nmatches>=0) {
+			if (nmatches==0) nmatches=(int)(sizeof(ovector)/sizeof(ovector[0]))/3*2; //only 2/3 of the vector is used by pcre_exec
+			if (nmatches==1 || !alias->SubPartern) { //no subpattern to replace
+				*url_ptr=alias->Alias;
+			} else {
+				repl_idx=0;
+				str=alias->Alias;
+				for (i=0 ; str[i] ; i++) {
+					// both the sed \1 and the perl $1 replacement operators are accepted
+					if ((str[i]=='\\' || str[i]=='$') && isdigit(str[i+1])) {
+						sub=str[++i]-'0';
+						if (sub>=1 && sub<=nmatches) {
+							/*
+							 * ovector[sub] is the start position of the match.
+							 * ovector[sub+1] is the end position of the match.
+							 */
+							sub<<=1;
+							if (repl_idx+ovector[sub+1]-ovector[sub]>=sizeof(Replacement)-1) break;
+							memcpy(Replacement+repl_idx,url+ovector[sub],ovector[sub+1]-ovector[sub]);
+							repl_idx+=ovector[sub+1]-ovector[sub];
+							continue;
+						}
+					}
+					if (repl_idx>=sizeof(Replacement)-1) break;
+					Replacement[repl_idx++]=str[i];
+				}
+				Replacement[repl_idx]='\0';
+				*url_ptr=Replacement;
+			}
+			return(true);
+		}
+	}
+	return(false);
+}
+#endif
+
+/*!
+Find the beginning of the URL beyond the scheme://
+
+\param url The url possibly containing a scheme.
+
+\return The beginning of the url beyond the scheme.
+*/
+const char *skip_scheme(const char *url)
+{
+	const char *str;
+	
+	/*
+	Skip any scheme:// at the beginning of the URL (see rfc2396 section 3.1).
+	The underscore is not part of the standard but is found in the squid logs as cache_object://.
+	*/
+	for (str=url ; *str && (isalnum(*str) || *str=='+' || *str=='-' || *str=='.' || *str=='_') ; str++);
+	if (str[0]==':' && str[1]=='/' && str[2]=='/') {
+		url=str+3;
+		while (*url=='/') url++;
+	}
+	return(url);
+}
+
 /*!
 Get the part of the URL necessary to generate the report.
 
@@ -609,26 +860,20 @@ const char *process_url(char *url,bool full_url)
 	char *str;
 	const char *start;
 	int type;
-	const char *address;
 	unsigned char ipv4[4];
 	unsigned short int ipv6[8];
 	const char *next;
 
-	/*
-	Remove any scheme:// at the beginning of the URL (see rfc2396 section 3.1).
-	The underscore is not part of the standard but is found in the squid logs as cache_object://.
-	*/
-	for (str=url ; *str && (isalnum(*str) || *str=='+' || *str=='-' || *str=='.' || *str=='_') ; str++);
-	if (*str==':' && str[1]=='/' && str[2]=='/') {
-		url=str+3;
-		while (*url=='/') url++;
-	}
-
-	start=url;
+	start=skip_scheme(url);
 	if (!full_url) {
-		for (str=url ; *str && *str!='/' && *str!='?' ; str++);
+		for (str=(char *)start ; *str && *str!='/' && *str!='?' ; str++);
 		*str='\0';
-		type=extract_address_mask(url,&address,ipv4,ipv6,NULL,&next);
+#ifdef USE_PCRE
+		if (FirstAliasRe) {
+			if (alias_url_regex(&start)) return(start);
+		}
+#endif
+		type=extract_address_mask(start,NULL,ipv4,ipv6,NULL,&next);
 		if (type==1) {
 			if (FirstAliasName)
 				start=alias_url_name(start,next);
