@@ -1,6 +1,6 @@
 /*
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
- *                                                            1998, 2011
+ *                                                            1998, 2012
  *
  * SARG donations:
  *      please look at http://sarg.sourceforge.net/donations.php
@@ -58,8 +58,8 @@ void useragent(void)
 	ipbefore[0]='\0';
 	namebefore[0]='\0';
 
-	sprintf(tmp3,"%s/squagent.unsort",tmp);
-	sprintf(tmp2,"%s/squagent.log",tmp);
+	sprintf(tmp3,"%s/squagent.int_unsort",tmp);
+	sprintf(tmp2,"%s/squagent.int_log",tmp);
 
 	if((fp_in=fopen(UserAgentLog,"r"))==NULL) {
 		debuga(_("(useragent) Cannot open file %s\n"),UserAgentLog);
@@ -148,8 +148,8 @@ void useragent(void)
 		exit(EXIT_FAILURE);
 	}
 
-	if (unlink(tmp3)) {
-		debuga(_("Cannot delete %s - %s\n"),tmp3,strerror(errno));
+	if (!KeepTempLog && unlink(tmp3)) {
+		debuga(_("Cannot delete \"%s\": %s\n"),tmp3,strerror(errno));
 		exit(EXIT_FAILURE);
 	}
 
@@ -231,8 +231,8 @@ void useragent(void)
 		exit(EXIT_FAILURE);
 	}
 
-	if (unlink(tmp2)) {
-		debuga(_("Cannot delete %s - %s\n"),tmp2,strerror(errno));
+	if (!KeepTempLog && unlink(tmp2)) {
+		debuga(_("Cannot delete \"%s\": %s\n"),tmp2,strerror(errno));
 		exit(EXIT_FAILURE);
 	}
 
@@ -279,8 +279,8 @@ void useragent(void)
 		exit(EXIT_FAILURE);
 	}
 
-	if (unlink(tmp3)) {
-		debuga(_("Cannot delete %s - %s\n"),tmp3,strerror(errno));
+	if (!KeepTempLog && unlink(tmp3)) {
+		debuga(_("Cannot delete \"%s\": %s\n"),tmp3,strerror(errno));
 		exit(EXIT_FAILURE);
 	}
 
@@ -300,8 +300,8 @@ void useragent(void)
 		exit(EXIT_FAILURE);
 	}
 
-	if (unlink(tmp2)) {
-		debuga(_("Cannot delete %s - %s\n"),tmp2,strerror(errno));
+	if (!KeepTempLog && unlink(tmp2)) {
+		debuga(_("Cannot delete \"%s\": %s\n"),tmp2,strerror(errno));
 		exit(EXIT_FAILURE);
 	}
 
@@ -333,8 +333,8 @@ void useragent(void)
 	if (fclose(fp_ht)==EOF)
 		debuga(_("Failed to close file %s - %s\n"),hfile,strerror(errno));
 
-	if (unlink(tmp3)) {
-		debuga(_("Cannot delete %s - %s\n"),tmp3,strerror(errno));
+	if (!KeepTempLog && unlink(tmp3)) {
+		debuga(_("Cannot delete \"%s\": %s\n"),tmp3,strerror(errno));
 		exit(EXIT_FAILURE);
 	}
 
