@@ -1,6 +1,6 @@
 /*
  * SARG Squid Analysis Report Generator      http://sarg.sourceforge.net
- *                                                            1998, 2011
+ *                                                            1998, 2012
  *
  * SARG donations:
  *      please look at http://sarg.sourceforge.net/donations.php
@@ -83,11 +83,11 @@ int getword(char *word, int limit, struct getwordstruct *gwarea, char stop)
 
 	for(x=0;((gwarea->current[x]) && (gwarea->current[x] != stop ));x++) {
 		if(x>=limit) {
-			printf("SARG: getword loop detected after %d bytes.\n",x);
-			printf("SARG: Line=\"%s\"\n",gwarea->beginning);
-			printf("SARG: Record=\"%s\"\n",gwarea->current);
-			printf("SARG: searching for \'x%x\'\n",stop);
-			//printf("SARG: Maybe you have a broken record or garbage in your access.log file.\n");
+			debuga(_("getword loop detected after %d bytes.\n"),x);
+			debuga(_("Line=\"%s\"\n"),gwarea->beginning);
+			debuga(_("Record=\"%s\"\n"),gwarea->current);
+			debuga(_("searching for \'x%x\'\n"),stop);
+			//debuga(_("Maybe you have a broken record or garbage in your access.log file.\n"));
 			word[(limit>0) ? limit-1 : 0]='\0';
 #if USE_GETWORD_BACKTRACE
 			getword_backtrace();
@@ -124,11 +124,11 @@ int getword_multisep(char *word, int limit, struct getwordstruct *gwarea, char s
 
 	for(x=0;((gwarea->current[x]) && (gwarea->current[x] != stop ));x++) {
 		if(x>=limit) {
-			printf("SARG: getword_multisep loop detected.\n");
-			printf("SARG: Line=\"%s\"\n",gwarea->beginning);
-			printf("SARG: Record=\"%s\"\n",gwarea->current);
-			printf("SARG: searching for \'x%x\'\n",stop);
-			//printf("SARG: Maybe you have a broken record or garbage in your access.log file.\n");
+			debuga(_("getword_multisep loop detected.\n"));
+			debuga(_("Line=\"%s\"\n"),gwarea->beginning);
+			debuga(_("Record=\"%s\"\n"),gwarea->current);
+			debuga(_("searching for \'x%x\'\n"),stop);
+			//debuga(_("Maybe you have a broken record or garbage in your access.log file.\n"));
 			if (limit>0) word[limit-1]='\0';
 #if USE_GETWORD_BACKTRACE
 			getword_backtrace();
@@ -151,11 +151,11 @@ int getword_skip(int limit, struct getwordstruct *gwarea, char stop)
 
 	for(x=0;(gwarea->current[x] && (gwarea->current[x] != stop ));x++) {
 		if(x>=limit) {
-			printf("SARG: getword_skip loop detected after %d bytes.\n",x);
-			printf("SARG: Line=\"%s\"\n",gwarea->beginning);
-			printf("SARG: Record=\"%s\"\n",gwarea->current);
-			printf("SARG: searching for \'x%x\'\n",stop);
-			//printf("SARG: Maybe you have a broken record or garbage in your access.log file.\n");
+			debuga(_("getword_skip loop detected after %d bytes.\n"),x);
+			debuga(_("Line=\"%s\"\n"),gwarea->beginning);
+			debuga(_("Record=\"%s\"\n"),gwarea->current);
+			debuga(_("searching for \'x%x\'\n"),stop);
+			//debuga(_("Maybe you have a broken record or garbage in your access.log file.\n"));
 #if USE_GETWORD_BACKTRACE
 			getword_backtrace();
 #endif
@@ -190,11 +190,11 @@ int getword_atoll(long long int *number, struct getwordstruct *gwarea, char stop
 		*number=(*number * 10) + digit;
 	}
 	if(gwarea->current[x] && gwarea->current[x]!=stop) {
-		printf("SARG: getword_atoll loop detected after %d bytes.\n",x);
-		printf("SARG: Line=\"%s\"\n",gwarea->beginning);
-		printf("SARG: Record=\"%s\"\n",gwarea->current);
-		printf("SARG: searching for \'x%x\'\n",stop);
-		//printf("SARG: Maybe you have a broken record or garbage in your access.log file.\n");
+		debuga(_("getword_atoll loop detected after %d bytes.\n"),x);
+		debuga(_("Line=\"%s\"\n"),gwarea->beginning);
+		debuga(_("Record=\"%s\"\n"),gwarea->current);
+		debuga(_("searching for \'x%x\'\n"),stop);
+		//debuga(_("Maybe you have a broken record or garbage in your access.log file.\n"));
 #if USE_GETWORD_BACKTRACE
 		getword_backtrace();
 #endif
@@ -229,11 +229,11 @@ int getword_atoi(int *number, struct getwordstruct *gwarea, char stop)
 		*number=(*number * 10) + digit;
 	}
 	if(gwarea->current[x] && gwarea->current[x]!=stop) {
-		printf("SARG: getword_atoi loop detected after %d bytes.\n",x);
-		printf("SARG: Line=\"%s\"\n",gwarea->beginning);
-		printf("SARG: Record=\"%s\"\n",gwarea->current);
-		printf("SARG: searching for \'x%x\'\n",stop);
-		//printf("SARG: Maybe you have a broken record or garbage in your access.log file.\n");
+		debuga(_("getword_atoi loop detected after %d bytes.\n"),x);
+		debuga(_("Line=\"%s\"\n"),gwarea->beginning);
+		debuga(_("Record=\"%s\"\n"),gwarea->current);
+		debuga(_("searching for \'x%x\'\n"),stop);
+		//debuga(_("Maybe you have a broken record or garbage in your access.log file.\n"));
 #if USE_GETWORD_BACKTRACE
 		getword_backtrace();
 #endif
@@ -315,7 +315,6 @@ void my_mkdir(const char *name)
 
 	if(!is_absolute(name)) {
 		debuga(_("Invalid path (%s). Please, use absolute paths only.\n"),name);
-		debuga(_("process aborted.\n"));
 		exit(EXIT_FAILURE);
 	}
 
@@ -330,7 +329,6 @@ void my_mkdir(const char *name)
 			if (access(w0, R_OK) != 0) {
 				if (mkdir(w0,0755)) {
 					debuga(_("Cannot create directory %s - %s\n"),w0,strerror(errno));
-					debuga(_("process aborted.\n"));
 					exit(EXIT_FAILURE);
 				}
 			}
@@ -342,7 +340,6 @@ void my_mkdir(const char *name)
 	if (access(name, R_OK) != 0) {
 		if (mkdir(name,0755)) {
 			debuga(_("Cannot create directory %s - %s\n"),name,strerror(errno));
-			debuga(_("process aborted.\n"));
 			exit(EXIT_FAILURE);
 		}
 	}
@@ -438,11 +435,11 @@ void name_month(char *month,int month_len)
 
 	for(x=0; x<z; x++)
 		if (getword_multisep(w,sizeof(w),&gwarea,',')<0) {
-			printf("SARG: Maybe you have a broken record or garbage in the names of the months.\n");
+			debuga(_("The internal list of month names is invalid. Please report this bug to the translator.\n"));
 			exit(EXIT_FAILURE);
 		}
 	if (getword_multisep(month,month_len,&gwarea,',')<0) {
-		printf("SARG: Maybe you have a broken record or garbage in the name of the months.\n");
+		debuga(_("The internal list of month names is invalid. Please report this bug to the translator.\n"));
 		exit(EXIT_FAILURE);
 	}
 }
@@ -1733,7 +1730,10 @@ void output_html_link(FILE *fp_ou,const char *url,int maxlen)
 		// this is an alias, no need for a A tag
 		output_html_string(fp_ou,url+1,100);
 	} else {
-		fputs("<a href=\"http://",fp_ou);
+		if (skip_scheme(url)==url)
+			fputs("<a href=\"http://",fp_ou);//no scheme in the url, assume http:// to make the link clickable
+		else
+			fputs("<a href=\"",fp_ou);//the scheme is in the url, no need to add one
 		output_html_url(fp_ou,url);
 		fputs("\">",fp_ou);
 		output_html_string(fp_ou,url,100);
@@ -1800,7 +1800,7 @@ char *get_param_value(const char *param,char *line)
 	return(line);
 }
 
-void unlinkdir(const char *dir,int contentonly)
+void unlinkdir(const char *dir,bool contentonly)
 {
 	struct stat st;
 	DIR *dirp;
@@ -1829,7 +1829,7 @@ void unlinkdir(const char *dir,int contentonly)
 		}
 		if (S_ISREG(st.st_mode)) {
 			if (unlink(dname)) {
-				debuga(_("cannot delete %s - %s\n"),dname,strerror(errno));
+				debuga(_("Cannot delete \"%s\": %s\n"),dname,strerror(errno));
 				exit(EXIT_FAILURE);
 			}
 		} else if (S_ISDIR(st.st_mode)) {
@@ -1842,18 +1842,143 @@ void unlinkdir(const char *dir,int contentonly)
 
 	if (!contentonly) {
 		if (rmdir(dir)) {
-			debuga(_("cannot delete %s - %s\n"),dir,strerror(errno));
+			debuga(_("Cannot delete \"%s\": %s\n"),dir,strerror(errno));
 			exit(EXIT_FAILURE);
 		}
 	}
 }
 
+/*!
+Delete every file from the temporary directory where sarg is told to store its
+temporary files.
+
+As any stray file left over by a previous run would be included in the report, we
+must delete every file from the temporary directory before we start processing the logs.
+
+But the temporary directory is given by the user either in the configuration file or
+on the command line. We check that the user didn't give a wrong directory by looking
+at the files stored in the directory. If a single file is not one of ours, we abort.
+
+\param dir The temporary directory to purge.
+*/
+void emptytmpdir(const char *dir)
+{
+	struct stat st;
+	DIR *dirp;
+	struct dirent *direntp;
+	int dlen;
+	int elen;
+	char dname[MAXLEN];
+	int err;
+	int i;
+	static const char *TmpExt[]=
+	{
+		".int_unsort",
+		".int_log",
+		".day",
+		"htmlrel.txt",
+		".user_unsort",
+		".user_log",
+		".utmp",
+		".ip"
+	};
+
+	dirp=opendir(dir);
+	if (!dirp) return;
+	
+	// make sure the temporary directory contains only our files
+	while ((direntp = readdir(dirp)) != NULL) {
+		if (direntp->d_name[0] == '.' && (direntp->d_name[1] == '\0' ||
+		    (direntp->d_name[1] == '.' && direntp->d_name[2] == '\0')))
+			continue;
+
+		// is it one of our files
+		dlen=strlen(direntp->d_name);
+		for (i=sizeof(TmpExt)/sizeof(TmpExt[0])-1 ; i>=0 ; i--) {
+			elen=strlen(TmpExt[i]);
+			if (dlen>=elen && strcasecmp(direntp->d_name+dlen-elen,TmpExt[i])==0) break;
+		}
+		if (i<0) {
+			debuga(_("Unknown file \"%s\" found in temporary directory \"%s\". It is not one of our files. "
+			"Please check the temporary directory you gave to sarg. Adjust the path to a safe "
+			"directory or manually delete the content of \"%s\"\n"),direntp->d_name,dir,dir);
+			exit(EXIT_FAILURE);
+		}
+		
+		if (snprintf(dname,sizeof(dname),"%s/%s",dir,direntp->d_name)>=sizeof(dname)) {
+			debuga(_("directory name to delete too long: %s/%s\n"),dir,direntp->d_name);
+			exit(EXIT_FAILURE);
+		}
+		
+#ifdef HAVE_LSTAT
+		err=lstat(dname,&st);
+#else
+		err=stat(dname,&st);
+#endif
+		if (err) {
+			debuga(_("cannot stat \"%s\"\n"),dname);
+			exit(EXIT_FAILURE);
+		}
+		if (S_ISDIR(st.st_mode)) {
+			unlinkdir(dname,0);
+		} else if (!S_ISREG(st.st_mode)) {
+			debuga(_("Unknown path type \"%s\". Check your temporary directory\n"),dname);
+			exit(EXIT_FAILURE);
+		}
+	}
+	rewinddir(dirp);
+
+	// now delete our files
+	while ((direntp = readdir(dirp)) != NULL) {
+		if (direntp->d_name[0] == '.' && (direntp->d_name[1] == '\0' ||
+		    (direntp->d_name[1] == '.' && direntp->d_name[2] == '\0')))
+			continue;
+
+		// is it one of our files
+		dlen=strlen(direntp->d_name);
+		for (i=sizeof(TmpExt)/sizeof(TmpExt[0])-1 ; i>=0 ; i--) {
+			elen=strlen(TmpExt[i]);
+			if (dlen>=elen && strcasecmp(direntp->d_name+dlen-elen,TmpExt[i])==0) break;
+		}
+		if (i<0) {
+			debuga(_("Unknown file \"%s\" found in temporary directory \"%s\". It is not one of our files. "
+			"Please check the temporary directory you gave to sarg. Adjust the path to a safe "
+			"directory or manually delete the content of \"%s\"\n"),direntp->d_name,dir,dir);
+			exit(EXIT_FAILURE);
+		}
+		
+		if (snprintf(dname,sizeof(dname),"%s/%s",dir,direntp->d_name)>=sizeof(dname)) {
+			debuga(_("directory name to delete too long: %s/%s\n"),dir,direntp->d_name);
+			exit(EXIT_FAILURE);
+		}
+#ifdef HAVE_LSTAT
+		err=lstat(dname,&st);
+#else
+		err=stat(dname,&st);
+#endif
+		if (err) {
+			debuga(_("cannot stat \"%s\"\n"),dname);
+			exit(EXIT_FAILURE);
+		}
+		if (S_ISREG(st.st_mode)) {
+			if (unlink(dname)) {
+				debuga(_("Cannot delete \"%s\": %s\n"),dname,strerror(errno));
+				exit(EXIT_FAILURE);
+			}
+		} else {
+			debuga(_("unknown path type %s\n"),dname);
+		}
+	}
+	closedir(dirp);
+}
+
 /*!
   Extract an url, IPv4 or IPv6 from a buffer. The IP addresses may end with a
   prefix size.
 
   \param buf The buffer to parse.
   \param text A pointer to set to the beginning of the string pattern. No terminating zero is inserted.
+              The pointer may be NULL.
   \param ipv4 A 4 bytes buffer to store the bytes of the IPv4 address.
   \param ipv6 A 8 short integers buffer to store the values of the IPv6 address.
   \param nbits The number of prefix bits for an IP address.
@@ -1876,7 +2001,9 @@ int extract_address_mask(const char *buf,const char **text,unsigned char *ipv4,u
 	int mask, max_mask;
 	int pad_pos;
 	int pad_len;
-	int bracket=false;
+	bool bracket=false;
+	bool port=false;
+	bool port_num=0;
 
 	// skip leading spaces and tabs
 	while (*buf && (*buf==' ' || *buf=='\t')) buf++;
@@ -1896,11 +2023,18 @@ int extract_address_mask(const char *buf,const char **text,unsigned char *ipv4,u
 	for (i=0 ; (unsigned char)buf[i]>' ' && buf[i]!='/' && buf[i]!='?' && (!bracket || buf[i]!=']') && ip_size ; i++) {
 		if (ip_size & 0x04) {
 			if (isdigit(buf[i])) {
-				value4=value4*10+(buf[i]-'0');
-				if (value4>0xFFU) ip_size&=~0x04;
+				if (port) {
+					port_num=port_num*10+(buf[i]-'0');
+					if (port_num>65535) ip_size&=~0x04;
+				} else {
+					value4=value4*10+(buf[i]-'0');
+					if (value4>0xFFU) ip_size&=~0x04;
+				}
 			} else if (buf[i]=='.' && addr_len<4) {
 				addr[addr_len++]=(unsigned short)(value4 & 0xFFU);
 				value4=0U;
+			} else if (!port && buf[i]==':') {
+				port=true;
 			} else {
 				ip_size&=~0x04;
 			}
@@ -1945,8 +2079,10 @@ int extract_address_mask(const char *buf,const char **text,unsigned char *ipv4,u
 			addr[addr_len++]=(unsigned short)(value6 & 0xFFFFU);
 	}
 	if (!ip_size) {
-		*text=buf;
-		if (bracket) (*text)--;
+		if (text) {
+			*text=buf;
+			if (bracket) (*text)--;
+		}
 		while ((unsigned char)buf[i]>' ') i++;
 		if (next) *next=buf+i;
 		return(1);
